@@ -1,4 +1,4 @@
-# admin-lib.pl
+# acl-lib.pl
 # Library for editing webmin users, passwords and access rights
 
 do '../web-lib.pl';
@@ -65,15 +65,21 @@ sub create_user
 local(%user, %miniserv, @mods);
 %user = %{$_[0]};
 &get_miniserv_config(\%miniserv);
+
+&lock_file($miniserv{'userfile'});
 open(PWFILE, ">> $miniserv{'userfile'}");
 print PWFILE "$user{'name'}:$user{'pass'}:$user{'sync'}:$user{'cert'}:",
 	$user{'allow'} ? "allow $user{'allow'}" :
 	$user{'deny'} ? "deny $user{'deny'}" : "","\n";
 close(PWFILE);
+&unlock_file($miniserv{'userfile'});
+
+&lock_file(&acl_filename());
 @mods = &list_modules();
 open(ACL, ">> ".&acl_filename());
 print ACL &acl_line(\%user, \@mods);
 close(ACL);
+&unlock_file(&acl_filename());
 
 delete($gconfig{"lang_".$user{'name'}});
 if ($user{'lang'}) {
@@ -105,6 +111,7 @@ local(%user, %miniserv, @pwfile, @acl, @mods, $_, $m);
 %user = %{$_[1]};
 &get_miniserv_config(\%miniserv);
 
+&lock_file($miniserv{'userfile'});
 open(PWFILE, $miniserv{'userfile'});
 @pwfile = <PWFILE>;
 close(PWFILE);
@@ -119,7 +126,9 @@ foreach (@pwfile) {
 	else { print PWFILE $_; }
 	}
 close(PWFILE);
+&unlock_file($miniserv{'userfile'});
 
+&lock_file(&acl_filename());
 @mods = &list_modules();
 open(ACL, &acl_filename());
 @acl = <ACL>;
@@ -132,6 +141,7 @@ foreach (@acl) {
 	else { print ACL $_; }
 	}
 close(ACL);
+&unlock_file(&acl_filename());
 
 delete($gconfig{"lang_".$_[0]});
 if ($user{'lang'}) {
@@ -157,6 +167,7 @@ sub delete_user
 {
 local($_, @pwfile, @acl, %miniserv);
 &get_miniserv_config(\%miniserv);
+&lock_file($miniserv{'userfile'});
 open(PWFILE, $miniserv{'userfile'});
 @pwfile = <PWFILE>;
 close(PWFILE);
@@ -165,7 +176,9 @@ foreach (@pwfile) {
 	if (!/^([^:\s]+):([^:\s]+)/ || $1 ne $_[0]) { print PWFILE $_; }
 	}
 close(PWFILE);
+&unlock_file($miniserv{'userfile'});
 
+&lock_file(&acl_filename());
 open(ACL, &acl_filename());
 @acl = <ACL>;
 close(ACL);
@@ -174,6 +187,7 @@ foreach (@acl) {
 	if (!/^(\S+):/ || $1 ne $_[0]) { print ACL $_; }
 	}
 close(ACL);
+&unlock_file(&acl_filename());
 
 unlink(map { "$config_directory/$_/$_[0].acl" } &list_modules());
 }
