@@ -23,7 +23,8 @@ if (&read_file("$module_config_directory/site", \%site)) {
 	foreach $m (keys %httpd_modules) {
 		do "./$m.pl";
 		}
-	foreach $d (split(/\s+/, $site{'defines'})) {
+	foreach $d (split(/\s+/, $site{'defines'}),
+		    split(/\s+/, $config{'defines'})) {
 		$httpd_defines{$d}++;
 		}
 	}
@@ -311,7 +312,7 @@ return \@conf;
 # to be updated in the directives array and in the file.
 sub save_directive
 {
-local($i, @old, $lref, $change, $len);
+local($i, @old, $lref, $change, $len, $v);
 @old = &find_directive_struct($_[0], $_[2]);
 for($i=0; $i<@old || $i<@{$_[1]}; $i++) {
 	$v = ${$_[1]}[$i];
@@ -647,14 +648,19 @@ if ($in{$_[0]} eq "") { return ( [ ] ); }
 else { return ( [ $in{$_[0]} ] ); }
 }
 
-# virtual_name(struct)
+# virtual_name(struct, [forlog])
 sub virtual_name
 {
 if ($_[0]) {
 	local $n = &find_directive("ServerName", $_[0]->{'members'});
-	return $n ? $n : $_[0]->{'value'};
+	if ($n) {
+		return $_[0]->{'value'} =~ /:(\d+)$/ ? "$n:$1" : $n;
+		}
+	else {
+		return $_[0]->{'value'};
+		}
 	}
-else { return "default server"; }
+else { return $_[1] ? "*" : $text{'default_serv'}; }
 }
 
 # dir_name(struct)
@@ -864,5 +870,21 @@ local $l = length($access{'dir'});
 return length($f) >= $l && substr($f, 0, $l) eq $access{'dir'};
 }
 
+sub lock_apache_files
+{
+local $conf = &get_config();
+foreach $f (&unique(map { $_->{'file'} } @$conf)) {
+	&lock_file($f);
+	}
+}
+
+sub unlock_apache_files
+{
+local $conf = &get_config();
+foreach $f (&unique(map { $_->{'file'} } @$conf)) {
+	&unlock_file($f);
+	}
+}
+
 1;
 
