@@ -151,7 +151,8 @@ return ( \@rv );
 @IndexOptions_d =
 	( "$text{'autoindex_18'}",
 	  "$text{'autoindex_19'}",
-	  "$text{'autoindex_20'}",
+	  "$text{'autoindex_20w'}",
+	  "$text{'autoindex_20h'}",
 	  "$text{'autoindex_21'}",
 	  "$text{'autoindex_22'}",
 	  "$text{'autoindex_23'}",
