@@ -25,12 +25,12 @@ for($i=0; $i<@regex; $i++) {
 	        "<td><input size=20 name=Browser_regex_$i value=\"%s\"></td>\n",
 	        $regex[$i];
 	$rv .= "<td>".&choice_input($case[$i], "Browser_case_$i", 1,
-	       "Yes,1", "No,0")."</td>\n";
+	       "$text{'browsermatch_7'},1", "$text{'browsermatch_8'},0")."</td>\n";
 	$rv .= sprintf
 	        "<td><input size=20 name=Browser_var_$i value=\"%s\"></td>\n",
 	        $var[$i];
 	$rv .= "<td>".&opt_input($val[$i], "Browser_val_$i",
-				 "Clear", 10)."</td>\n";
+				 "$text{'browsermatch_6'}", 10)."</td>\n";
 	$rv .= "</tr>\n";
 	}
 $rv .= "</table>\n";
