@@ -70,7 +70,7 @@ $rv = [	[ 'AccessFileName', 0, 5, 'virtual', undef, 5 ],
 	[ 'ScoreBoardFile', 0, 9, 'global', 1.2 ],
 	[ 'SendBufferSize', 0, 1, 'global' ],
 	[ 'ServerAdmin', 0, 1, 'virtual' ],
-	[ 'ServerAlias', 1, 1, 'virtual', 1.2 ],
+	[ 'ServerAlias', 1, 1, 'virtual virtualonly', 1.2 ],
 	[ 'ServerName', 0, 1, 'virtual' ],
 	[ 'ServerPath', 0, 5, 'virtual' ],
 	[ 'ServerType', 0, 9, 'global' ],
@@ -195,7 +195,7 @@ sub rlimit_input
 {
 local(@v, $rv);
 @v = split(/\s+/, $_[2]->{'value'});
-$rv = sprintf "<input type=radio name=$_[0]_mode value=0 %s> Default<br>\n",
+$rv = sprintf "<input type=radio name=$_[0]_mode value=0 %s> $text{'Default'}<br>\n",
 	@v ? "" : "checked";
 $rv .= sprintf "<input type=radio name=$_[0]_mode value=1 %s>\n",
 	@v == 1 ? "checked" : "";
@@ -262,14 +262,14 @@ $inp .=	"<table border>\n".
 for($i=0; $i<@blist+1; $i++) {
 	$inp .= sprintf
 		"<tr><td><input type=radio name=BindAddress_def_$i value=2 %s>".
-		" None <input type=radio name=BindAddress_def_$i value=1 %s>".
-		" All <input type=radio name=BindAddress_def_$i value=0 %s> ".
+		" $text{'core_201'} <input type=radio name=BindAddress_def_$i value=1 %s>".
+		" $text{'core_202'} <input type=radio name=BindAddress_def_$i value=0 %s> ".
 		"<input name=BindAddress_$i size=20 value=\"%s\"></td>",
 		$blist[$i] ? "" : "checked",
 		$blist[$i] eq "*" ? "checked" : "",
 		$blist[$i] && $blist[$i] ne "*" ? "checked" : "",
 		$blist[$i] eq "*" ? "" : $blist[$i];
-	$inp .= "<td>".&opt_input($plist[$i], "port_$i", "$text{'core_34'}", 5)."</td>";
+	$inp .= "<td>".&opt_input($plist[$i], "Port_$i", "$text{'core_34'}", 5)."</td>";
 	}
 $inp .= "</table>\n";
 return (2, $text{'core_35'}, $inp);
@@ -320,8 +320,8 @@ $kref = $_[0]; $mref = $_[1];
 if ($_[2]->{'version'} >= 1.2) {
 	# two separate directives for keep-alives
 	$inp = sprintf
-		"<input type=radio name=KeepAlive_def value=0 %s> None\n".
-		"<input type=radio name=KeepAlive_def value=1 %s> Default\n".
+		"<input type=radio name=KeepAlive_def value=0 %s> $text{'core_203'}\n".
+		"<input type=radio name=KeepAlive_def value=1 %s> $text{'core_204'}\n".
 		"<input type=radio name=KeepAlive_def value=2 %s> ".
 		"<input name=KeepAlive size=5 value=\"%s\">",
 		$kref->{'value'} =~ /off/i ? "checked" : "",
@@ -332,8 +332,8 @@ if ($_[2]->{'version'} >= 1.2) {
 else {
 	# only one directive
 	$inp = sprintf
-		"<input type=radio name=KeepAlive_def value=0 %s> None\n".
-		"<input type=radio name=KeepAlive_def value=1 %s> Default\n".
+		"<input type=radio name=KeepAlive_def value=0 %s> $text{'core_205'}\n".
+		"<input type=radio name=KeepAlive_def value=1 %s> $text{'core_206'}\n".
 		"<input type=radio name=KeepAlive_def value=2 %s> ".
 		"<input name=KeepAlive size=5 value=\"%s\">",
 		$kref->{'value'} eq "0" ? "checked" : "",
@@ -729,22 +729,22 @@ sub edit_require
 local($rv, $mode, $list);
 $_[0]->{'value'} =~ /^(\S+)\s*(.*)$/; $mode = $1; $list = $2;
 $rv = sprintf
-      "<input type=radio name=require_mode value=0 %s> Default<br>\n",
+      "<input type=radio name=require_mode value=0 %s> $text{'default'}<br>\n",
       $mode ? "" : "checked";
 $rv .= sprintf
-      "<input type=radio name=require_mode value=1 %s> Only these users:\n",
+      "<input type=radio name=require_mode value=1 %s> $text{'core_198'}:\n",
       $mode eq "user" ? "checked" : "";
 $rv .= sprintf
       "<input name=require_user size=20 value=\"%s\"><br>\n",
       $mode eq "user" ? $list : "";
 $rv .= sprintf
-      "<input type=radio name=require_mode value=2 %s> Only these groups:\n",
+      "<input type=radio name=require_mode value=2 %s> $text{'core_199'}:\n",
       $mode eq "group" ? "checked" : "";
 $rv .= sprintf
       "<input name=require_group size=20 value=\"%s\"><br>\n",
       $mode eq "group" ? $list : "";
 $rv .= sprintf
-      "<input type=radio name=require_mode value=3 %s> All valid users<br>\n",
+      "<input type=radio name=require_mode value=3 %s> $text{'core_200'}:<br>\n",
       $mode eq "valid-user" ? "checked" : "";
 return (1, $text{'core_130'}, $rv);
 }
@@ -943,6 +943,7 @@ for($i=0; $i<$len; $i++) {
 	$v = $_[0]->[$i]->{'value'};
 	if ($v =~ /^(\d+)\s+((http|ftp|gopher):\S+)$/)
 		{ $code = $1; $type = 0; $url = $2; }
+	elsif ($v =~ /^(\d+)\s+(\/.*)$/) { $code = $1; $type = 0; $url = $2; }
 	elsif ($v =~ /^(\d+)\s+"?(.*)$/) { $code = $1; $type = 1; $url = $2; }
 	else { $code = ""; $type = 0; $url = ""; }
 	$rv .= "<tr $cb>\n";
