@@ -28,7 +28,8 @@ $rv .= "<tr> <td><b>$text{'mod_auth_2'}</b></td> <td>".
 
 $rv .= "<tr> <td><b>$text{'mod_auth_4'}</b></td> <td>".
        &choice_input($_[2]->{'value'}, "AuthAuthoritative", "",
-       "Yes,off", "No,on", "Default,")."</td> </tr>\n";
+       "$text{'yes'},off", "$text{'no'},on", "$text{'default'},").
+       "</td> </tr>\n";
 $rv .= "</table></td></tr></table>\n";
 return (2, "$text{'mod_auth_5'}", $rv);
 }
@@ -38,8 +39,8 @@ $in{'AuthUserFile_def'} || &allowed_auth_file($in{'AuthUserFile'}) ||
 	&error($text{'mod_auth_6'});
 $in{'AuthGroupFile_def'} || &allowed_auth_file($in{'AuthGroupFile'}) ||
 	&error($text{'mod_auth_7'});
-return (&parse_opt("AuthUserFile", '^\S+$', "Invalid user text file name") ,
-        &parse_opt("AuthGroupFile", '^\S+$', "Invalid group text file name") ,
+return (&parse_opt("AuthUserFile", '^\S+$', $text{'mod_auth_11'}) ,
+        &parse_opt("AuthGroupFile", '^\S+$', $text{'mod_auth_12'}) ,
         &parse_choice("AuthAuthoritative", ""));
 }
 
