@@ -15,25 +15,26 @@ local($rv, $uf, $gf);
 $uf = $_[0] ? 1 : 0; $gf = $_[1] ? 1 : 0;
 $rv = "<table border><tr><td><table>\n";
 
-$rv .= "<tr> <td><b>User DBM file</b></td> <td>".
-       &opt_input($_[0]->{'value'}, "AuthDBMUserFile", "Default", 25).
+$rv .= "<tr> <td><b>$text{'mod_auth_dbm_0'}</b></td> <td>".
+       &opt_input($_[0]->{'value'}, "AuthDBMUserFile", $text{'default'}, 25).
        &file_chooser_button("AuthDBMUserFile", 0)."</td></tr>\n";
 
-$rv .= "<tr> <td><b>Group DBM file</b></td> <td>".
-       &opt_input($_[1]->{'value'}, "AuthDBMGroupFile", "Default", 25).
+$rv .= "<tr> <td><b>$text{'mod_auth_dbm_2'}</b></td> <td>".
+       &opt_input($_[1]->{'value'}, "AuthDBMGroupFile", $text{'default'}, 25).
        &file_chooser_button("AuthDBMGroupFile", 0)."</td></tr>\n";
 
-$rv .= "<tr> <td><b>Pass failures to next module?</b></td> <td>".
+$rv .= "<tr> <td><b>$text{'mod_auth_dbm_4'}</b></td> <td>".
        &choice_input($_[2]->{'value'}, "AuthDBMAuthoritative", "",
-       "Yes,off", "No,on", "Default,")."</td> </tr>\n";
+       "$text{'yes'},off", "$text{'no'},on", "$text{'default'},").
+       "</td> </tr>\n";
 $rv .= "</table></td></tr></table>\n";
-return (2, "DBM file authentication", $rv);
+return (2, $text{'mod_auth_dbm_8'}, $rv);
 }
 sub save_AuthDBMUserFile_AuthDBMGroupFile_AuthDBMAuthoritative
 {
 local(@rv);
-@rv = (&parse_opt("AuthDBMUserFile", '^\S+$', "Invalid user DBM file name") ,
-       &parse_opt("AuthDBMGroupFile", '^\S+$', "Invalid group DBM file name") ,
+@rv = (&parse_opt("AuthDBMUserFile", '^\S+$', $text{'mod_auth_dbm_9'}) ,
+       &parse_opt("AuthDBMGroupFile", '^\S+$', $text{'mod_auth_dbm_10'}) ,
        &parse_choice("AuthDBMAuthoritative", ""));
 return @rv;
 }
