@@ -14,9 +14,9 @@ return &make_directives($rv, $_[0], "mod_cern_meta");
 
 sub edit_MetaFiles
 {
-return (1, "Process header metafiles",
+return (1, "$text{'mod_cern_meta_0'}",
         &choice_input($_[0]->{'value'}, "MetaFiles", "off",
-        "Yes,on", "No,off"));
+        "$text{'mod_cern_meta_1'},on", "$text{'mod_cern_meta_2'},off"));
 }
 sub save_MetaFiles
 {
@@ -25,22 +25,22 @@ return &parse_choice("MetaFiles", "off");
 
 sub edit_MetaDir
 {
-return (1, "Subdirectory for header metafiles",
-        &opt_input($_[0]->{'value'}, "MetaDir", "Default", 8));
+return (1, "$text{'mod_cern_meta_3'}",
+        &opt_input($_[0]->{'value'}, "MetaDir", "$text{'mod_cern_meta_4'}", 8));
 }
 sub save_MetaDir
 {
-return &parse_opt("MetaDir", '^\S+$', "Invalid metafile subdirectory name");
+return &parse_opt("MetaDir", '^\S+$', "$text{'mod_cern_meta_5'}");
 }
 
 sub edit_MetaSuffix
 {
-return (1, "File suffix for header metafiles",
-        &opt_input($_[0]->{'value'}, "MetaSuffix", "Default", 15));
+return (1, "$text{'mod_cern_meta_6'}",
+        &opt_input($_[0]->{'value'}, "MetaSuffix", "$text{'mod_cern_meta_7'}", 15));
 }
 sub save_MetaSuffix
 {
-return &parse_opt("MetaSuffix", '^\S+$', "Invalid metafile suffix");
+return &parse_opt("MetaSuffix", '^\S+$', "$text{'mod_cern_meta_8'}");
 }
 
 1;
