@@ -14,9 +14,9 @@ return ("server-parsed");
 
 sub edit_XBitHack
 {
-return (2, "Process includes on files with execute bit?",
+return (2, "$text{'mod_include_0'}",
         &choice_input($_[0]->{'value'}, "XBitHack", "",
-        "No,off", "Yes,on", "Yes and set last-modified date,full", "Default,"));
+        "$text{'mod_include_1'},off", "$text{'mod_include_2'},on", "$text{'mod_include_3'},full", "$text{'mod_include_4'},"));
 }
 sub save_XBitHack
 {
