@@ -27,7 +27,7 @@ if ($_[1]->{'version'} >= 1.3) {
 		$rv .= "<tr $cb> <td><input name=LogFormat_nick_$i size=10 ".
 		       "value=\"$nick[$i]\"></td>\n";
 		$rv .= "<td><input name=LogFormat_fmt_$i size=40 ".
-		       "value=\"$fmt[$i]\"></td> </tr>\n";
+		       "value='$fmt[$i]'></td> </tr>\n";
 		}
 	$rv .= "</table>\n";
 	return (2, "$text{'mod_log_config_4'}", $rv);
