@@ -42,12 +42,12 @@ for($i=0; $i<@head; $i++) {
 	        "<td><input size=10 name=SetEnvIf_regex_$i value=\"%s\"></td>\n",
 	        $regex[$i];
 	$rv .= "<td>".&choice_input($case[$i], "SetEnvIf_case_$i", 1,
-	       "Yes,1", "No,1")."</td>\n";
+	       "$text{'mod_setenvif_10'},1", "$text{'mod_setenvif_11'},1")."</td>\n";
 	$rv .= sprintf
 	        "<td><input size=20 name=SetEnvIf_var_$i value=\"%s\"></td>\n",
 	        $val[$i];
 	$rv .= "<td>".&opt_input($val[$i], "SetEnvIf_var_$i",
-	       "Clear", 10)."</td>\n";
+	       "$text{'mod_setenvif_9'}", 10)."</td>\n";
 	$rv .= "</tr>\n";
 	}
 $rv .= "</table>\n";
