@@ -111,7 +111,7 @@ while($i < @tok) {
 			$dir{'class'} = "IN";
 			$i++;
 			}
-		if (!$dir{'name'}) {
+		if ($dir{'name'} eq '') {
 			$#rv >= 0 ||
 				&error(&text('efirst', $lnum[$i]+1, $file));
 			$dir{'name'} = $rv[$#rv]->{'name'};
@@ -312,7 +312,7 @@ return	$a1 < $1 ? -1 :
 # Converts an address like 4.3.2.1.in-addr.arpa. to 1.2.3.4
 sub arpa_to_ip
 {
-if ($_[0] =~ /^([\d\.]+)\.in-addr\.arpa/i) {
+if ($_[0] =~ /^([\d\-\.]+)\.in-addr\.arpa/i) {
 	return join('.',reverse(split(/\./, $1)));
 	}
 return $_[0];
@@ -322,7 +322,7 @@ return $_[0];
 # Converts an IP address like 1.2.3.4 to 4.3.2.1.in-addr.arpa.
 sub ip_to_arpa
 {
-if ($_[0] =~ /^([\d\.]+)$/) {
+if ($_[0] =~ /^([\d\-\.]+)$/) {
 	return join('.',reverse(split(/\./,$1))).".in-addr.arpa.";
 	}
 return $_[0];
