@@ -46,6 +46,7 @@ sub save_command
 {
 local $c = $_[0];
 local $a;
+&lock_file("$module_config_directory/$c->{'id'}.cmd");
 open(FILE, ">$module_config_directory/$c->{'id'}.cmd");
 print FILE $c->{'cmd'},"\n";
 print FILE $c->{'desc'},"\n";
@@ -55,13 +56,16 @@ foreach $a (@{$c->{'args'}}) {
 		   $a->{'opts'},":",$a->{'quote'},":",$a->{'desc'},"\n";
 	}
 close(FILE);
+&unlock_file("$module_config_directory/$c->{'id'}.cmd");
 }
 
 # delete_command(&command)
 sub delete_command
 {
 local $c = $_[0];
+&lock_file("$module_config_directory/$c->{'id'}.cmd");
 unlink("$module_config_directory/$c->{'id'}.cmd");
+&unlock_file("$module_config_directory/$c->{'id'}.cmd");
 }
 
 sub can_run_command
