@@ -82,8 +82,9 @@ if ((-d "/proc/ide" || -d "/proc/scsi") && -r "/proc/partitions") {
 			}
 		}
 	close(SCSI);
-	@pscsi = grep { /Type:\s+Direct-Access/ } @pscsi;
+	@pscsi = grep { /Type:\s+Direct-Access/i } @pscsi;
 	for($i=0; $i<@scsi; $i++) {
+		next if ($pscsi[$i] =~ /USB-FDU/i);     # skip USB floppies!
 		&open_fdisk("-l", $scsi[$i]);
 		while(<$fh>) {
 			if (/Disk\s+([^ :]+):\s+(\d+)\s+\S+\s+(\d+)\s+\S+\s+(\d+)/) {
@@ -678,6 +679,8 @@ else { return " $_[2] $in{$_[0]}"; }
 	    "msdos","MS-DOS",
 	    "vfat","Windows 95");
 
+@space_type = ( '1', '4', '5', 'b', 'c', 'e', '83' );
+
 # can_edit_disk(device)
 sub can_edit_disk
 {
@@ -687,3 +690,12 @@ foreach (split(/\s+/, $access{'disks'})) {
 return 0;
 }
 
+# disk_space(device)
+# Returns the amount of total and free space for some filesystem, or an
+# empty array if not appropriate.
+sub disk_space
+{
+`df -k $_[0]` =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/ || return ();
+return ($1, $2);
+}
+
