@@ -14,7 +14,7 @@ public class FileManager extends Applet
 	// top buttons
 	CbButton open_b, view_b, edit_b, refresh_b, props_b,
 		 copy_b, cut_b, paste_b, delete_b, new_b, upload_b, mkdir_b,
-		 makelink_b, rename_b;
+		 makelink_b, rename_b, share_b;
 
 	// Directory tree
 	Hierarchy dirs;
@@ -35,6 +35,10 @@ public class FileManager extends Applet
 					 "Sep", "Oct", "Nov", "Dec" };
 	String accroot[];
 	Hashtable lang = new Hashtable();
+	Hashtable stab = new Hashtable(),
+		  ntab = new Hashtable();
+	boolean sambamode;
+	int nfsmode;
 
 	public void init()
 	{
@@ -51,6 +55,34 @@ public class FileManager extends Applet
 		lang.put(l[i].substring(0, eq), l[i].substring(eq+1));
 		}
 
+	// list samba file shares
+	String s[] = get_text("list_shares.cgi");
+	if (s[0].equals("1")) {
+		for(int i=1; i<s.length; i++) {
+			SambaShare ss = new SambaShare(s[i]);
+			stab.put(ss.path, ss);
+			}
+		sambamode = true;
+		}
+
+	// list NFS exports
+	String e[] = get_text("list_exports.cgi");
+	nfsmode = e.length == 0 ? 0 : Integer.parseInt(e[0]);
+	if (nfsmode != 0) {
+		for(int i=1; i<e.length; i++) {
+			if (nfsmode == 1) {
+				// Linux export
+				LinuxExport le = new LinuxExport(e[i]);
+				ntab.put(le.path, le);
+				}
+			else if (nfsmode == 2) {
+				// Solaris share
+				DFSAdminExport de = new DFSAdminExport(e[i]);
+				ntab.put(de.path, de);
+				}
+			}
+		}
+
 	// create button panel
 	BorderPanel top = new BorderPanel(2);
 	top.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
@@ -75,6 +107,8 @@ public class FileManager extends Applet
 		top2.add(makelink_b = make_button("makelink.gif",
 					text("top_new")));
 	top2.add(rename_b = make_button("rename.gif", text("top_rename")));
+	if (sambamode || nfsmode != 0)
+		top2.add(share_b = make_button("share.gif", text("top_share")));
 	top.add(top2);
 	top.add(new Label(""));
 
@@ -169,7 +203,10 @@ public class FileManager extends Applet
 	// Create file rows
 	for(int i=0; i<fl.length; i++) {
 		Object row[] = rows[i+1] = new Object[6];
-		row[0] = get_image(RemoteFile.tmap[fl[i].type]);
+		if (fl[i].shared())
+			row[0] = get_image("sdir.gif");
+		else
+			row[0] = get_image(RemoteFile.tmap[fl[i].type]);
 		row[1] = fl[i].name;
 		if (fl[i].size < 1000)
 			row[2] = spad(fl[i].size, 5)+" b";
@@ -368,6 +405,11 @@ public class FileManager extends Applet
 		if (f == null) return;
 		new RenameWindow(this, f);
 		}
+	else if (b == share_b) {
+		// Open a window for editing sharing options
+		if (f == null || f.type != RemoteFile.DIR) return;
+		new SharingWindow(f, this);
+		}
 	}
 
 	// Returns the object for some directory, or null if not found.
@@ -497,7 +539,7 @@ public class FileManager extends Applet
 	catch(Exception e) { }
 	}
 
-	String urlize(String s)
+	static String urlize(String s)
 	{
 	StringBuffer rv = new StringBuffer();
 	for(int i=0; i<s.length(); i++) {
@@ -528,6 +570,7 @@ public class FileManager extends Applet
 	public String text(String k, String p[])
 	{
 	String rv = (String)lang.get(k);
+	if (rv == null) rv = "???";
 	for(int i=0; i<p.length; i++) {
 		int idx = rv.indexOf("$"+(i+1));
 		if (idx != -1)
@@ -624,7 +667,7 @@ class FileNode extends HierarchyNode
 	{
 	this.file = file;
 	parent = file.parent;
-	im = parent.get_image("dir.gif");
+	im = parent.get_image(file.shared() ? "sdir.gif" : "dir.gif");
 	ch = new Vector();
 	text = file.name;
 	parent.nodemap.put(file, this);
@@ -753,6 +796,13 @@ class RemoteFile
 			nlist[j++] = list[i];
 	list = nlist;
 	}
+
+	boolean shared()
+	{
+	return type == DIR &&
+	       (parent.stab.get(path) != null ||
+	        parent.ntab.get(path) != null);
+	}
 }
 
 class EditorWindow extends FixedFrame implements CbButtonCallback
@@ -1509,3 +1559,485 @@ class OverwriteWindow extends FixedFrame implements CbButtonCallback
 	}
 }
 
+class SambaShare
+{
+	String path;
+	boolean available;
+	boolean writable;
+	int guest;
+	String comment;
+
+	SambaShare(String l)
+	{
+	StringSplitter tok = new StringSplitter(l, ':');
+	path = tok.nextToken();
+	available = tok.nextToken().equals("1");
+	writable = tok.nextToken().equals("1");
+	guest = Integer.parseInt(tok.nextToken());
+	comment = tok.nextToken();
+	}
+
+	SambaShare(String p, boolean a, boolean w, int g, String c)
+	{
+	path = p;
+	available = a;
+	writable = w;
+	guest = g;
+	comment = c;
+	}
+
+	String params()
+	{
+	return "path="+FileManager.urlize(path)+
+	       "&available="+(available ? 1 : 0)+
+	       "&writable="+(writable ? 1 : 0)+
+	       "&guest="+guest+
+	       "&comment="+FileManager.urlize(comment);
+	}
+}
+
+class DFSAdminExport
+{
+	String path;
+	String desc;
+	String ro, rw, root;
+
+	DFSAdminExport(String l)
+	{
+	StringSplitter tok = new StringSplitter(l, ':');
+	path = tok.nextToken();
+	ro = tok.nextToken();
+	rw = tok.nextToken();
+	root = tok.nextToken();
+	desc = tok.nextToken();
+	}
+
+	DFSAdminExport(String p, String d, String ro, String rw, String root)
+	{
+	path = p;
+	desc = d;
+	this.ro = ro;
+	this.rw = rw;
+	this.root = root;
+	}
+
+	String[] split(String s)
+	{
+	StringTokenizer stok = new StringTokenizer(s, " ");
+	String rv[] = new String[stok.countTokens()];
+	for(int i=0; i<rv.length; i++)
+		rv[i] = stok.nextToken();
+	return rv;
+	}
+
+	String params()
+	{
+	return "path="+FileManager.urlize(path)+
+	       "&ro="+FileManager.urlize(ro)+
+	       "&rw="+FileManager.urlize(rw)+
+	       "&root="+FileManager.urlize(root)+
+	       "&desc="+FileManager.urlize(desc);
+	}
+}
+
+class LinuxExport
+{
+	String path;
+	String host[];
+	boolean ro[];
+	int squash[];
+
+	LinuxExport(String l)
+	{
+	StringSplitter tok = new StringSplitter(l, ':');
+	path = tok.nextToken();
+	int c = tok.countTokens() / 3;
+	host = new String[c];
+	ro = new boolean[c];
+	squash = new int[c];
+	for(int i=0; tok.hasMoreTokens(); i++) {
+		host[i] = tok.nextToken();
+		ro[i] = tok.nextToken().equals("1");
+		squash[i] = Integer.parseInt(tok.nextToken());
+		}
+	}
+
+	LinuxExport(String p, String h[], String r[], String s[])
+	{
+	path = p;
+	}
+
+	String params()
+	{
+	String rv = "path="+FileManager.urlize(path)+
+		    "&count="+host.length;
+	for(int i=0; i<host.length; i++) {
+		rv += "&host"+i+"="+FileManager.urlize(host[i]);
+		rv += "&ro"+i+"="+(ro[i] ? 1 : 0);
+		rv += "&squash"+i+"="+squash[i];
+		}
+	return rv;
+	}
+}
+
+class SharingWindow extends FixedFrame implements CbButtonCallback
+{
+	CbButton save_b, cancel_b;
+	RemoteFile file;
+	FileManager filemgr;
+	SambaShare sshare;
+	DFSAdminExport dexport;
+	LinuxExport lexport;
+	Checkbox samba_on, samba_off;
+	Checkbox writable_on, writable_off;
+	Checkbox available_on, available_off;
+	Checkbox guest_on, guest_off, guest_only;
+	TextField comment;
+
+	TextField desc;
+	Checkbox nfs_on, nfs_off;
+	TextField rwhosts, rohosts, roothosts;
+	Checkbox rw[] = new Checkbox[3], ro[] = new Checkbox[3],
+		 root[] = new Checkbox[3];
+
+	TextField host[];
+	Choice lro[], squash[];
+
+	SharingWindow(RemoteFile f, FileManager p)
+	{
+	file = f; filemgr = p;
+	setTitle(filemgr.text("share_title", file.path));
+	sshare = (SambaShare)filemgr.stab.get(file.path);
+	Object nshare = filemgr.ntab.get(file.path);
+	if (filemgr.nfsmode == 1)
+		lexport = (LinuxExport)nshare;
+	else if (filemgr.nfsmode == 2)
+		dexport = (DFSAdminExport)nshare;
+
+	// setup UI
+	setLayout(new BorderLayout());
+	Panel samba = new Panel(), sl = new Panel(), sr = new Panel();
+	samba.setLayout(new BorderLayout());
+	Panel st = new Panel();
+	st.setLayout(new GridLayout(2, 1));
+	CheckboxGroup sg = new CheckboxGroup();
+	st.add(samba_off = new Checkbox(filemgr.text("share_soff"), sg, 
+				       sshare == null));
+	st.add(samba_on = new Checkbox(filemgr.text("share_son"), sg,
+					sshare != null));
+	samba.add("North", st);
+
+	Panel stop = new LinedPanel(filemgr.text("share_sheader"));
+	setup_leftright(stop, sl, sr);
+
+	comment = new TextField(sshare == null ? "" : sshare.comment, 25);
+	add_item(filemgr.text("share_comment"), comment, sl, sr);
+
+	Panel ap = new Panel();
+	ap.setLayout(new GridLayout(1, 0));
+	CheckboxGroup ag = new CheckboxGroup();
+	ap.add(available_on = new Checkbox(filemgr.text("yes"), ag,
+					  sshare == null || sshare.available));
+	ap.add(available_off = new Checkbox(filemgr.text("no"), ag,
+					  sshare != null && !sshare.available));
+	add_item(filemgr.text("share_available"), ap, sl, sr);
+
+	Panel wp = new Panel();
+	wp.setLayout(new GridLayout(1, 0));
+	CheckboxGroup wg = new CheckboxGroup();
+	wp.add(writable_on = new Checkbox(filemgr.text("yes"), wg,
+					  sshare == null || sshare.writable));
+	wp.add(writable_off = new Checkbox(filemgr.text("no"), wg,
+					   sshare != null && !sshare.writable));
+	add_item(filemgr.text("share_writable"), wp, sl, sr);
+
+	Panel gp = new Panel();
+	gp.setLayout(new GridLayout(1, 0));
+	CheckboxGroup gg = new CheckboxGroup();
+	gp.add(guest_only = new Checkbox(filemgr.text("share_only"), gg,
+				sshare != null && sshare.guest == 2));
+	gp.add(guest_on = new Checkbox(filemgr.text("yes"), gg,
+				sshare == null || sshare.guest == 1));
+	gp.add(guest_off = new Checkbox(filemgr.text("no"), gg,
+			 	sshare != null && sshare.guest == 0));
+	add_item(filemgr.text("share_guest"), gp, sl, sr);
+
+	samba.add("Center", stop);
+
+	// Setup NFS UI
+	Panel nfs = new Panel(), nl = new Panel(), nr = new Panel();
+	nfs.setLayout(new BorderLayout());
+	Panel nt = new Panel();
+	nt.setLayout(new GridLayout(2, 1));
+	CheckboxGroup ng = new CheckboxGroup();
+	nt.add(nfs_off = new Checkbox(filemgr.text("share_noff"), ng, 
+				      nshare == null));
+	nt.add(nfs_on = new Checkbox(filemgr.text("share_non"), ng,
+				     nshare != null));
+	nfs.add("North", nt);
+
+	Panel ntop = new LinedPanel(filemgr.text("share_nheader"));
+	setup_leftright(ntop, nl, nr);
+	if (filemgr.nfsmode == 1) {
+		// Linux export mode
+		nl.setLayout(new GridLayout(0, 1, 2, 2));
+		nr.setLayout(new GridLayout(0, 1, 2, 2));
+		nl.add(new Label(filemgr.text("share_host")));
+		nr.add(new Label(filemgr.text("share_opts")));
+		int c = lexport==null ? 0 : lexport.host.length;
+		host = new TextField[c+1];
+		lro = new Choice[c+1];
+		squash = new Choice[c+1];
+		for(int i=0; i<c; i++) {
+			host[i] = new TextField(lexport.host[i], 20);
+			lro[i] = robox(lexport.ro[i]);
+			squash[i] = squashbox(lexport.squash[i]);
+			nl.add(host[i]);
+			nr.add(opts_panel(lro[i], squash[i]));
+			}
+		host[c] = new TextField("", 20);
+		lro[c] = robox(false);
+		squash[c] = squashbox(1);
+		nl.add(host[c]);
+		nr.add(opts_panel(lro[c], squash[c]));
+		}
+	else if (filemgr.nfsmode == 2) {
+		// Solaris share mode
+		desc = new TextField(dexport == null ? "" : dexport.desc, 25);
+		add_item(filemgr.text("share_desc"), desc, nl, nr);
+
+		rohosts = add_hosts(filemgr.text("share_ro"),
+				    dexport == null ? "-" : dexport.ro,
+				    ro, nl, nr);
+		rwhosts = add_hosts(filemgr.text("share_rw"),
+				    dexport == null ? "-" : dexport.rw,
+				    rw, nl, nr);
+		roothosts = add_hosts(filemgr.text("share_root"),
+				    dexport == null ? "-" : dexport.root,
+				    root, nl, nr);
+		root[1].getParent().remove(root[1]);
+		}
+	else if (filemgr.nfsmode == 3) {
+		}
+	nfs.add("Center", ntop);
+
+	// Add the appropriate tabs
+	if (filemgr.sambamode && filemgr.nfsmode != 0) {
+		TabbedPanel tab = new TabbedPanel();
+		tab.addItem(filemgr.text("share_samba"), samba);
+		tab.addItem(filemgr.text("share_nfs"), nfs);
+		add("Center", tab);
+		}
+	else if (filemgr.sambamode)
+		add("Center", samba);
+	else if (filemgr.nfsmode != 0)
+		add("Center", nfs);
+
+	// Create save and cancel buttons
+	Panel bot = new Panel();
+	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
+	bot.add(save_b = new CbButton(filemgr.get_image("save.gif"),
+				      filemgr.text("save"),
+				      CbButton.LEFT, this));
+	bot.add(cancel_b = new CbButton(filemgr.get_image("cancel.gif"),
+					filemgr.text("cancel"),
+					CbButton.LEFT, this));
+	add("South", bot);
+	pack();
+	show();
+	}
+
+	public void click(CbButton b)
+	{
+	if (b == save_b) {
+		// Update samba settings on server
+		if (sshare != null && samba_on.getState()) {
+			// Updating share
+			sshare.available = available_on.getState();
+			sshare.writable = writable_on.getState();
+			sshare.guest = guest_only.getState() ? 2 :
+				       guest_on.getState() ? 1 : 0;
+			sshare.comment = comment.getText();
+			String rv[] = filemgr.get_text(
+				"save_share.cgi?"+sshare.params());
+			}
+		else if (sshare != null) {
+			// Deleting share
+			String rv[] = filemgr.get_text(
+				"save_share.cgi?delete=1&"+sshare.params());
+			filemgr.stab.remove(sshare.path);
+			}
+		else if (samba_on.getState()) {
+			// Creating share
+			sshare = new SambaShare(file.path,
+						available_on.getState(),
+						writable_on.getState(),
+						guest_only.getState() ? 2 :
+						guest_on.getState() ? 1 : 0,
+						comment.getText());
+			filemgr.stab.put(sshare.path, sshare);
+			String rv[] = filemgr.get_text(
+				"save_share.cgi?new=1&"+sshare.params());
+			}
+
+		// Update NFS settings on server
+		if (filemgr.nfsmode == 1) {
+			if (lexport != null && nfs_on.getState()) {
+				// Updating export
+				export_options(lexport);
+				String rv[] = filemgr.get_text(
+					"save_export.cgi?"+lexport.params());
+				}
+			else if (lexport != null) {
+				// Deleting export
+				String rv[] = filemgr.get_text(
+				  "save_export.cgi?delete=1&"+lexport.params());
+				filemgr.ntab.remove(lexport.path);
+				}
+			else if (nfs_on.getState()) {
+				// Creating export
+				lexport = new LinuxExport(file.path, null,
+							  null, null);
+				export_options(lexport);
+				String rv[] = filemgr.get_text(
+				  "save_export.cgi?new=1&"+lexport.params());
+				filemgr.ntab.put(lexport.path, lexport);
+				}
+			}
+		else if (filemgr.nfsmode == 2) {
+			if (dexport != null && nfs_on.getState()) {
+				// Updating share
+				dexport.desc = desc.getText();
+				dexport.ro = ro[0].getState() ? "-" :
+					     ro[1].getState() ? "" :
+					     rohosts.getText();
+				dexport.rw = rw[0].getState() ? "-" :
+					     rw[1].getState() ? "" :
+					     rwhosts.getText();
+				dexport.root = root[0].getState() ? "-" :
+					       roothosts.getText();
+				String rv[] = filemgr.get_text(
+					"save_export.cgi?"+dexport.params());
+				}
+			else if (dexport != null) {
+				// Deleting share
+				String rv[] = filemgr.get_text(
+				  "save_export.cgi?delete=1&"+dexport.params());
+				filemgr.ntab.remove(dexport.path);
+				}
+			else if (nfs_on.getState()) {
+				// Creating new share
+				dexport = new DFSAdminExport(file.path,
+					desc.getText(),
+					ro[0].getState() ? "-" :
+					ro[1].getState() ? "" :
+					rohosts.getText(),
+					rw[0].getState() ? "-" :
+					rw[1].getState() ? "" :
+					rwhosts.getText(),
+					root[0].getState() ? "-" :
+					roothosts.getText());
+				String rv[] = filemgr.get_text(
+				    "save_export.cgi?new=1&"+dexport.params());
+				filemgr.ntab.put(dexport.path, dexport);
+				}
+			}
+		else if (filemgr.nfsmode == 3) {
+			}
+
+		filemgr.show_files(filemgr.showing_files);
+		dispose();
+		}
+	else if (b == cancel_b)
+		dispose();
+	}
+
+	void setup_leftright(Panel m, Panel l, Panel r)
+	{
+	m.setLayout(new BorderLayout());
+	Panel p = new Panel();
+	p.setLayout(new BorderLayout());
+	p.add("West", l);
+	p.add("Center", r);
+	l.setLayout(new GridLayout(0, 1));
+	r.setLayout(new GridLayout(0, 1));
+	m.add("North", p);
+	}
+
+	void add_item(String t, Component c, Panel l, Panel r)
+	{
+	l.add(new Label(t));
+	Panel p = new Panel();
+	p.setLayout(new BorderLayout());
+	p.add("West", c);
+	r.add(p);
+	}
+
+	TextField add_hosts(String name, String v, Checkbox cb[],
+			    Panel l, Panel r)
+	{
+	Panel p = new Panel();
+	p.setLayout(new GridLayout(1, 3));
+	CheckboxGroup g = new CheckboxGroup();
+	p.add(cb[0] = new Checkbox(filemgr.text("share_none"), g,
+				   v.equals("-")));
+	p.add(cb[1] = new Checkbox(filemgr.text("share_all"), g,
+				   v.length() == 0));
+	p.add(cb[2] = new Checkbox(filemgr.text("share_listed"), g,
+				   v.length() > 1));
+	add_item(name, p, l, r);
+	TextField t = new TextField(v.equals("-") ? "" : v, 25);
+	add_item("", t, l, r);
+	return t;
+	}
+
+	Choice squashbox(int s)
+	{
+	Choice rv = new Choice();
+	rv.addItem(filemgr.text("share_s0"));
+	rv.addItem(filemgr.text("share_s1"));
+	rv.addItem(filemgr.text("share_s2"));
+	rv.select(s);
+	return rv;
+	}
+
+	Choice robox(boolean r)
+	{
+	Choice rv = new Choice();
+	rv.addItem(filemgr.text("share_lrw"));
+	rv.addItem(filemgr.text("share_lro"));
+	rv.select(r ? 1 : 0);
+	return rv;
+	}
+
+	Panel opts_panel(Component ro, Component squash)
+	{
+	Panel p = new Panel();
+	p.setLayout(new BorderLayout());
+	System.out.println("adding "+ro+" "+squash);
+	p.add("West", ro);
+	p.add("East", squash);
+	return p;
+	}
+
+	void export_options(LinuxExport e)
+	{
+	int c = 0;
+	for(int i=0; i<host.length; i++)
+		if (host[i].getText().length() > 0)
+			c++;
+	e.host = new String[c];
+	e.ro = new boolean[c];
+	e.squash = new int[c];
+	for(int i=0,j=0; i<host.length; i++) {
+		if (host[i].getText().trim().length() > 0) {
+			e.host[j] = host[i].getText();
+			e.ro[j] = lro[i].getSelectedIndex() == 1;
+			e.squash[j] = squash[i].getSelectedIndex();
+			j++;
+			}
+		}
+	}
+
+}
+
