@@ -47,16 +47,15 @@ if ($access{'uid'}) {
 # can_access(file)
 sub can_access
 {
-foreach $r (split(/\s+/, $access{'root'})) {
-	return 1 if ($r eq "/");
-	local $lf = length($_[0]);
-	local $lr = length($r);
-	if ($lf > $lr) {
-		return 1 if (substr($_[0], 0, $lr) eq $r);
-		}
-	else {
-		return 1 if (substr($r, 0, $lf) eq $_[0]);
+local @f = grep { $_ ne '' } split(/\//, $_[0]);
+DIR: foreach $r (split(/\s+/, $access{'root'})) {
+	return 1 if ($r eq '/' || $_[0] eq '/');
+	local @a = grep { $_ ne '' } split(/\//, $r);
+	local $i;
+	for($i=0; $i<@a && $i<@f; $i++) {
+		next DIR if ($a[$i] ne $f[$i]);
 		}
+	return 1;
 	}
 return 0;
 }
