@@ -17,6 +17,9 @@ while(<SERVICES>) {
 	chop; s/#.*$//g;
 	if (/^(\S+)\s+([0-9]+)\/(\S+)\s*(.*)$/) {
 		push(@rv, [ $l, $1, $2, $3, $4, scalar(@rv) ]);
+		if ($config{'ipv6'}) {
+			push(@rv, [ $l, $1, $2, $3.'6', $4, scalar(@rv) ]);
+			}
 		}
 	$l++;
 	}
@@ -28,8 +31,17 @@ return @rv;
 # Add a new service to the list
 sub create_service
 {
+local $p = $_[2];
+if ($config{'ipv6'} && $p =~ /^(\S+)6$/) {
+	# don't add the service if it is already there
+	foreach $s (&list_services()) {
+		return if ($s->[1] eq $_[0] && $s->[2] == $_[1] &&
+			   $s->[3] eq $1);
+		}
+	$p =~ s/6$//;
+	}
 open(SERVICES, ">> $config{services_file}");
-print SERVICES "$_[0]\t$_[1]/$_[2]",$_[3] ? "\t$_[3]\n" : "\n";
+print SERVICES "$_[0]\t$_[1]/$p",$_[3] ? "\t$_[3]\n" : "\n";
 close(SERVICES);
 system("$config{'put_services_command'}") if ($config{'put_services_command'});
 }
@@ -77,7 +89,10 @@ $l = 0;
 open(INET, $config{'inetd_conf_file'});
 while(<INET>) {
 	chop;
-	if (/^(#+|#<off>#)?\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\/\S+|internal)\s*(.*)$/) { push(@rv, [ $l, !$1, 0, $2, $3, $4, $5, $6, $7, $8 ]); $rv[$#rv]->[2] = ($2 =~ /\//); }
+	if (/^(#+|#<off>#)?\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\/\S+|internal)\s*(.*)$/) {
+		push(@rv, [ $l, !$1, 0, $2, $3, $4, $5, $6, $7, $8 ]);
+		$rv[$#rv]->[2] = ($2 =~ /\//);
+		}
 	$l++;
 	}
 close(INET);
@@ -138,7 +153,7 @@ open(RPC, $config{rpc_file});
 while(<RPC>) {
 	chop; s/#.*$//g;
 	if (/^(\S+)\s+(\d+)\s*(.*)$/) {
-		push(@rv, [ $l, $1, $2, $3, $4, scalar(@rv) ]);
+		push(@rv, [ $l, $1, $2, $3, scalar(@rv) ]);
 		}
 	$l++;
 	}
@@ -197,13 +212,35 @@ while(<PROT>) {
 	if (!/\S/) { next; }
 	/^(\S+)\s+/;
 	push(@rv, $1);
+	if ($config{'ipv6'}) {
+		if ($1 eq 'tcp') { push(@rv, 'tcp6'); }
+		elsif ($1 eq 'udp') { push(@rv, 'udp6'); }
+		}
 	}
 close(PROT);
-return @rv;
+return &unique(@rv);
+}
+
+sub lock_inetd_files
+{
+&lock_file($config{'inetd_conf_file'}, 0, 1);
+&lock_file($config{'services_file'}, 0, 1);
+&lock_file($config{'protocols_file'}, 0, 1);
+&lock_file($config{'rpc_file'}, 0, 1);
+}
+
+sub unlock_inetd_files
+{
+&unlock_file($config{'inetd_conf_file'});
+&unlock_file($config{'services_file'});
+&unlock_file($config{'protocols_file'});
+&unlock_file($config{'rpc_file'});
 }
 
 %prot_name = ("ip", "Internet Protocol",
 	      "tcp", "Transmission Control Protocol",
-	      "udp", "User Datagram Protocol");
+	      "udp", "User Datagram Protocol",
+	      "tcp6", "Transmission Control Protocol IPv6",
+	      "udp6", "User Datagram Protocol IPv6");
 
 1;
