@@ -20,7 +20,7 @@ close(STAT);
 return &unique(@rv);
 }
 
-# get_printer(name)
+# get_printer(name, [nostatus])
 # Returns a reference to an associative array of printer details
 sub get_printer
 {
@@ -53,16 +53,18 @@ if (!$prn{'enabled'} && $body1 =~ /^\s+(.*)/) {
 	$prn{'enabled_why'} = $1 eq "reason unknown" ? "" : $1;
 	}
 
-# request availability
-$avl = `lpstat -a$prn{'name'} 2>&1`;
-if ($avl =~ /^\S+\s+not accepting.*\n\s+(.*)/) {
-	$prn{'accepting'} = 0;
-	$prn{'accepting_why'} = $1;
-	if ($prn{'accepting_why'} eq "reason unknown") {
-		$prn{'accepting_why'} = "";
+if (!$_[1]) {
+	# request availability
+	$avl = `lpstat -a$prn{'name'} 2>&1`;
+	if ($avl =~ /^\S+\s+not accepting.*\n\s+(.*)/) {
+		$prn{'accepting'} = 0;
+		$prn{'accepting_why'} = $1;
+		if ($prn{'accepting_why'} eq "reason unknown") {
+			$prn{'accepting_why'} = "";
+			}
 		}
+	else { $prn{'accepting'} = 1; }
 	}
-else { $prn{'accepting'} = 1; }
 
 # request connection
 $con = `lpstat -v$prn{'name'} 2>&1`;
@@ -223,13 +225,14 @@ else {
 	}
 
 # stop scheduler
-$out = `lpshut 2>&1`;
+$out = &backquote_logged("lpshut 2>&1");
 
 # call lpadmin
-$out = `$cmd 2>&1`;
+$out = &backquote_logged("$cmd 2>&1");
 if ($?) { &error("lpsched failed : <pre>$out</pre>"); }
 
 ## Link to windows webmin driver
+&lock_file("$hpux_iface_path/$prn{'name'}");
 if ($wdrv) {
 	`rm $hpux_iface_path/$prn{'name'}`;
 	`ln -s $drivers_directory/$prn{'name'}.smb $hpux_iface_path/$prn{'name'}`;
@@ -246,11 +249,14 @@ if ($prn{'iface'} eq "$drivers_directory/$prn{'name'}" && !$wdrv) {
 	`rm $hpux_iface_path/$prn{'name'}`;
 	`ln -s $drivers_directory/$prn{'name'} $hpux_iface_path/$prn{'name'}`;
 	}
+&unlock_file("$hpux_iface_path/$prn{'name'}");
+&lock_file("$config{'model_path'}/$dummy");
 `rm $config{'model_path'}/$dummy`;
+&unlock_file("$config{'model_path'}/$dummy");
 
 # start scheduler
 if ($scheduler) {
-	$out = `lpsched 2>&1`;
+	$out = &backquote_logged("lpsched 2>&1");
 	if ($?) { &error("lpsched failed : <pre>$out</pre>"); }
 	}
 
@@ -266,7 +272,7 @@ local(%prn, $cmd, $out);
 
 # make the default
 if ($prn{'default'}) {
-	$out = `lpadmin -d$prn{'name'} 2>&1`;
+	$out = &backquote_logged("lpadmin -d$prn{'name'} 2>&1");
 	if ($?) { &error("Failed to set default : <pre>$out</pre>"); }
 	}
 
@@ -275,14 +281,14 @@ if ($prn{'enabled'}) { $cmd = "enable $prn{'name'}"; }
 elsif ($prn{'enabled_why'})
 	{ $cmd = "enable $prn{'name'} ; disable -r\"$prn{'enabled_why'}\" $prn{'name'}"; }
 else { $cmd = "enable $prn{'name'} ; disable $prn{'name'}"; }
-$out = `$cmd 2>&1`;
+$out = &backquote_logged("$cmd 2>&1");
 
 # accepting or rejecting requests
 if ($prn{'accepting'}) { $cmd = "accept $prn{'name'}"; }
 elsif ($prn{'accepting_why'})
 	{ $cmd = "accept $prn{'name'} ; reject -r\"$prn{'accepting_why'}\" $prn{'name'}"; }
 else { $cmd = "accept $prn{'name'} ; reject $prn{'name'}"; }
-$out = `$cmd 2>&1`;
+$out = &backquote_logged("$cmd 2>&1");
 }
 
 # delete_printer(name)
@@ -293,19 +299,19 @@ local($out, $scheduler);
 $scheduler = &sched_running();
 
 # delete print jobs
-$out = `cancel $_[0] -a 2>1`;
+$out = &backquote_logged("cancel $_[0] -a 2>1");
 if ($?) { &error("cancel failed : <pre>$out</pre>"); }
 
 # stop scheduler
-$out = `lpshut 2>&1`;
+$out = &backquote_logged("lpshut 2>&1");
 
 # call lpadmin
-$out = `lpadmin -x$_[0] 2>&1`;
+$out = &backquote_logged("lpadmin -x$_[0] 2>&1");
 if ($?) { &error("lpadmin failed : <pre>$out</pre>"); }
 
 # start scheduler
 if ($scheduler) {
-	$out = `lpsched 2>&1`;
+	$out = &backquote_logged("lpsched 2>&1");
 	if ($?) { &error("lpsched failed : <pre>$out</pre>"); }
 	}
 }
@@ -315,7 +321,7 @@ if ($scheduler) {
 sub cancel_job
 {
 local($out);
-$out = `cancel $_[1] 2>&1`;
+$out = &backquote_logged("cancel $_[1] 2>&1");
 if ($?) { &error("cancel failed : <pre>$out</pre>"); }
 sleep(1);
 }
@@ -333,7 +339,7 @@ else { return 1; }
 # Start lpsched
 sub start_sched
 {
-local $out = `lpsched 2>&1`;
+local $out = &backquote_logged("lpsched 2>&1");
 if ($?) { &error("failed to start lpsched : <tt>$out</tt>"); }
 }
 
@@ -341,7 +347,7 @@ if ($?) { &error("failed to start lpsched : <tt>$out</tt>"); }
 # Stop the running lpsched process
 sub stop_sched
 {
-local $out = `lpshut 2>&1`;
+local $out = &backquote_logged("lpshut 2>&1");
 if ($?) { &error("lpshut failed : <tt>$out</tt>"); }
 }
 
