@@ -13,6 +13,7 @@ foreach $l (&list_printcap()) {
 return @rv;
 }
 
+# get_printer(name, [nostatus])
 sub get_printer
 {
 local($l, %prn, @w, @n, $w, %cap, @jobs);
@@ -30,10 +31,12 @@ foreach $l (&list_printcap()) {
 		$prn{'rqueue'} = $l->{'rp'};
 		$prn{'msize'} = $l->{'mx'};
 
-		# call lpc to get status
-		$out = `lpc status $prn{'name'} 2>&1`;
-		$prn{'accepting'} = ($out =~ /queuing is enabled/);
-		$prn{'enabled'} = ($out =~ /printing is enabled/);
+		if (!$_[1]) {
+			# call lpc to get status
+			$out = `lpc status $prn{'name'} 2>&1`;
+			$prn{'accepting'} = ($out =~ /queuing is enabled/);
+			$prn{'enabled'} = ($out =~ /printing is enabled/);
+			}
 
 		return \%prn;
 		}
@@ -74,10 +77,14 @@ sub create_printer
 {
 local(%cap);
 $cap{'sd'} = "$config{'spool_dir'}/$_[0]->{'name'}";
+&lock_file($cap{'sd'});
 mkdir($cap{'sd'}, 0755);
+&unlock_file($cap{'sd'});
+&lock_file($config{'printcap_file'});
 open(CAP, ">> $config{'printcap_file'}");
 print CAP &make_printcap($_[0], \%cap),"\n";
 close(CAP);
+&unlock_file($config{'printcap_file'});
 &apply_status($_[0]);
 }
 
@@ -85,6 +92,7 @@ close(CAP);
 sub modify_printer
 {
 local(@old, $o, $old, @cap);
+&lock_file($config{'printcap_file'});
 @old = &list_printcap();
 foreach $o (@old) {
 	$o->{'name'} =~ /^([^\|]+)/;
@@ -103,6 +111,7 @@ splice(@cap, $old->{'line'},
 open(CAP, "> $config{'printcap_file'}");
 print CAP @cap;
 close(CAP);
+&unlock_file($config{'printcap_file'});
 &apply_status($_[0]);
 }
 
@@ -110,6 +119,7 @@ close(CAP);
 sub delete_printer
 {
 local(@old, $o, $old, @cap);
+&lock_file($config{'printcap_file'});
 @old = &list_printcap();
 foreach $o (@old) {
 	$o->{'name'} =~ /^([^\|]+)/;
@@ -127,6 +137,7 @@ splice(@cap, $old->{'line'}, $old->{'eline'} - $old->{'line'} + 1);
 open(CAP, "> $config{'printcap_file'}");
 print CAP @cap;
 close(CAP);
+&unlock_file($config{'printcap_file'});
 }
 
 # cancel_job(printer, job)
@@ -134,7 +145,7 @@ close(CAP);
 sub cancel_job
 {
 local($out);
-$out = `lprm -P$_[0] $_[1] 2>&1`;
+$out = &backquote_logged("lprm -P$_[0] $_[1] 2>&1");
 if ($?) { &error("lprm failed : $out"); }
 }
 
@@ -212,13 +223,13 @@ sub apply_status
 local($out);
 $out = `lpc status $prn{'name'} 2>&1`;
 if ($prn{'enabled'} && $out !~ /printing is enabled/)
-	{ `lpc up $prn{'name'}`; }
+	{ &backquote_logged("lpc up $prn{'name'}"); }
 elsif (!$prn{'enabled'} && $out =~ /printing is enabled/)
-	{ `lpc down $prn{'name'}`; }
+	{ &backquote_logged("lpc down $prn{'name'}"); }
 if ($prn{'accepting'} && $out !~ /queuing is enabled/)
-	{ `lpc enable $prn{'name'}`; }
+	{ &backquote_logged("lpc enable $prn{'name'}"); }
 elsif (!$prn{'accepting'} && $out =~ /queuing is enabled/)
-	{ `lpc disable $prn{'name'}`; }
+	{ &backquote_logged("lpc disable $prn{'name'}"); }
 }
 
 # sched_running()
@@ -234,7 +245,7 @@ return 0;
 # Start lpsched
 sub start_sched
 {
-local $out = `lpd 2>&1`;
+local $out = &backquote_logged("lpd 2>&1");
 if ($?) { &error("failed to start lpd : <tt>$out</tt>"); }
 }
 
@@ -242,10 +253,15 @@ if ($?) { &error("failed to start lpd : <tt>$out</tt>"); }
 # Stop the running lpsched process
 sub stop_sched
 {
-kill('TERM', $_[0]) || &error("Failed to stop lpd : $!");
+&kill_logged('TERM', $_[0]) || &error("Failed to stop lpd : $!");
 }
 
-@device_files = ("/dev/lpt0", "/dev/cua00", "/dev/cua01");
+if ($gconfig{'os_type'} eq 'openbsd') {
+	@device_files = ("/dev/lpt0", "/dev/cua00", "/dev/cua01");
+	}
+else {
+	@device_files = ("/dev/lpt0", "/dev/cuaa0", "/dev/cuaa1");
+	}
 @device_names = ("Parallel port", "Serial Port 1 (COM1)",
 		 "Serial Port 2 (COM2)");
 
