@@ -15,7 +15,7 @@ close(STAT);
 return &unique(@rv);
 }
 
-# get_printer(name)
+# get_printer(name, [nostatus])
 # Returns a reference to an associative array of printer details
 sub get_printer
 {
@@ -58,16 +58,18 @@ if ($body =~ /Users (allowed|denied):\n((\s+\S+\n)+)/) {
 	else { $prn{'deny'} = \@l; }
 	}
 
-# request availability
-$avl = `lpstat -a $prn{'name'} 2>&1`;
-if ($avl =~ /^\S+\s+not accepting.*\n\s+(.*)/) {
-	$prn{'accepting'} = 0;
-	$prn{'accepting_why'} = $1;
-	if ($prn{'accepting_why'} eq "unknown reason") {
-		$prn{'accepting_why'} = "";
+if (!$_[1]) {
+	# request availability
+	$avl = `lpstat -a $prn{'name'} 2>&1`;
+	if ($avl =~ /^\S+\s+not accepting.*\n\s+(.*)/) {
+		$prn{'accepting'} = 0;
+		$prn{'accepting_why'} = $1;
+		if ($prn{'accepting_why'} eq "unknown reason") {
+			$prn{'accepting_why'} = "";
+			}
 		}
+	else { $prn{'accepting'} = 1; }
 	}
-else { $prn{'accepting'} = 1; }
 
 # request connection
 $con = `lpstat -v $prn{'name'} 2>&1`;
@@ -151,7 +153,8 @@ local(%prn, $cmd, $out);
 
 # call lpsystem if needed
 if ($prn{'rhost'}) {
-	$out = `lpsystem -t $prn{'rtype'} $prn{'rhost'} 2>&1`;
+	$out = &backquote_logged(
+		"lpsystem -t $prn{'rtype'} $prn{'rhost'} 2>&1");
 	if ($?) { &error("lpsystem failed : <pre>$out</pre>"); }
 	}
 
@@ -160,11 +163,11 @@ $cmd = "lpadmin -p $prn{'name'} -D \"$prn{'desc'}\"";
 if ($prn{'allow_all'}) { $cmd .= " -u allow:all"; }
 elsif ($prn{'deny_all'}) { $cmd .= " -u deny:all"; }
 elsif ($prn{'allow'}) {
-	system("lpadmin -p $prn{'name'} -u deny:all >/dev/null 2>&1");
+	&system_logged("lpadmin -p $prn{'name'} -u deny:all >/dev/null 2>&1");
 	$cmd .= " -u allow:".join(',', @{$prn{'allow'}});
 	}
 elsif ($prn{'deny'}) {
-	system("lpadmin -p $prn{'name'} -u allow:all >/dev/null 2>&1");
+	&system_logged("lpadmin -p $prn{'name'} -u allow:all >/dev/null 2>&1");
 	$cmd .= " -u deny:".join(',', @{$prn{'deny'}});
 	}
 if ($prn{'dev'}) {
@@ -181,12 +184,12 @@ if ($prn{'dev'}) {
 else { $cmd .= " -s $prn{'rhost'}!$prn{'rqueue'}"; }
 @ctype = @{$prn{'ctype'}};
 if (@ctype) { $cmd .= " -I ".join(',' , @ctype); }
-$out = `cd / ; $cmd 2>&1`;
+$out = &backquote_logged("cd / ; $cmd 2>&1");
 if ($?) { &error("lpadmin failed : <pre>$out ($cmd)</pre>"); }
 
 # make the default
 if ($prn{'default'}) {
-	$out = `cd / ; lpadmin -d $prn{'name'} 2>&1`;
+	$out = &backquote_logged("cd / ; lpadmin -d $prn{'name'} 2>&1");
 	if ($?) { &error("Failed to set default : <pre>$out</pre>"); }
 	}
 
@@ -195,14 +198,16 @@ if ($prn{'enabled'}) { $cmd = "enable $prn{'name'}"; }
 elsif ($prn{'enabled_why'})
 	{ $cmd = "enable $prn{'name'} ; disable -r \"$prn{'enabled_why'}\" $prn{'name'}"; }
 else { $cmd = "enable $prn{'name'} ; disable $prn{'name'}"; }
-$out = `$cmd 2>&1`;
+$out = &backquote_logged("$cmd 2>&1");
 
 # accepting or rejecting requests
 if ($prn{'accepting'}) { $cmd = "accept $prn{'name'}"; }
-elsif ($prn{'accepting_why'})
-	{ $cmd = "accept $prn{'name'} ; reject -r \"$prn{'accepting_why'}\" $prn{'name'}"; }
+elsif ($prn{'accepting_why'}) {
+	$cmd = "accept $prn{'name'} ; ".
+	       "reject -r \"$prn{'accepting_why'}\" $prn{'name'}";
+	}
 else { $cmd = "accept $prn{'name'} ; reject $prn{'name'}"; }
-$out = `$cmd 2>&1`;
+$out = &backquote_logged("$cmd 2>&1");
 }
 
 # delete_printer(name)
@@ -210,7 +215,7 @@ $out = `$cmd 2>&1`;
 sub delete_printer
 {
 local($out);
-$out = `lpadmin -x $_[0] 2>&1`;
+$out = &backquote_logged("lpadmin -x $_[0] 2>&1");
 if ($?) { &error("lpadmin failed : <pre>$out</pre>"); }
 }
 
@@ -219,7 +224,7 @@ if ($?) { &error("lpadmin failed : <pre>$out</pre>"); }
 sub cancel_job
 {
 local($out);
-$out = `cancel $_[1] 2>&1`;
+$out = &backquote_logged("cancel $_[1] 2>&1");
 if ($?) { &error("cancel failed : <pre>$out</pre>"); }
 sleep(1);
 }
@@ -237,7 +242,7 @@ return 0;
 # Start lpsched
 sub start_sched
 {
-local $out = `/usr/lib/lp/lpsched 2>&1`;
+local $out = &backquote_logged("/usr/lib/lp/lpsched 2>&1");
 if ($?) { &error("failed to start lpsched : <tt>$out</tt>"); }
 }
 
@@ -245,12 +250,12 @@ if ($?) { &error("failed to start lpsched : <tt>$out</tt>"); }
 # Stop the running lpsched process
 sub stop_sched
 {
-local $out = `lpshut 2>&1`;
+local $out = &backquote_logged("lpshut 2>&1");
 if ($?) { &error("lpshut failed : <tt>$out</tt>"); }
 }
 
 
-@device_files = ("/dev/bpp0", "/dev/eccp0", "/dev/term/a",
+@device_files = ("/dev/bpp0", "/dev/ecpp0", "/dev/term/a",
 		 "/dev/term/b", "/dev/null");
 @device_names = ("Parallel port", "Ultra 5/10 Parallel port", "Serial port A",
 		 "Serial port B", "Null Device");
