@@ -16,7 +16,7 @@ close(STAT);
 return &unique(@rv);
 }
 
-# get_printer(name)
+# get_printer(name, [nostatus])
 # Returns a reference to an associative array of printer details
 sub get_printer
 {
@@ -58,16 +58,18 @@ if ($body =~ /Users (allowed|denied):\n((\s+\S+\n)+)/) {
 	else { $prn{'deny'} = \@l; }
 	}
 
-# request availability
-$avl = `lpstat -a $prn{'name'} 2>&1`;
-if ($avl =~ /^\S+\s+not accepting.*\n\s+(.*)/) {
-	$prn{'accepting'} = 0;
-	$prn{'accepting_why'} = $1;
-	if ($prn{'accepting_why'} eq "unknown reason") {
-		$prn{'accepting_why'} = "";
+if (!$_[1]) {
+	# request availability
+	$avl = `lpstat -a $prn{'name'} 2>&1`;
+	if ($avl =~ /^\S+\s+not accepting.*\n\s+(.*)/) {
+		$prn{'accepting'} = 0;
+		$prn{'accepting_why'} = $1;
+		if ($prn{'accepting_why'} eq "unknown reason") {
+			$prn{'accepting_why'} = "";
+			}
 		}
+	else { $prn{'accepting'} = 1; }
 	}
-else { $prn{'accepting'} = 1; }
 
 # request connection
 $con = `lpstat -v $prn{'name'} 2>&1`;
@@ -174,7 +176,8 @@ local(%prn, $cmd, $out);
 
 # call lpsystem if needed
 if ($prn{'rhost'}) {
-	$out = `lpsystem -t $prn{'rtype'} $prn{'rhost'} 2>&1`;
+	$out = &backquote_logged(
+		"lpsystem -t $prn{'rtype'} $prn{'rhost'} 2>&1");
 	if ($?) { &error("lpsystem failed : <pre>$out</pre>"); }
 	}
 
@@ -199,12 +202,12 @@ if ($prn{'dev'}) {
 else { $cmd .= " -s $prn{'rhost'}!$prn{'rqueue'}"; }
 @ctype = @{$prn{'ctype'}};
 if (@ctype) { $cmd .= " -I ".join(',' , @ctype); }
-$out = `$cmd 2>&1`;
+$out = &backquote_logged("$cmd 2>&1");
 if ($?) { &error("lpadmin failed : <pre>$out</pre>"); }
 
 # make the default
 if ($prn{'default'}) {
-	$out = `lpadmin -d $prn{'name'} 2>&1`;
+	$out = &backquote_logged("lpadmin -d $prn{'name'} 2>&1");
 	if ($?) { &error("Failed to set default : <pre>$out</pre>"); }
 	}
 
@@ -213,14 +216,14 @@ if ($prn{'enabled'}) { $cmd = "enable $prn{'name'}"; }
 elsif ($prn{'enabled_why'})
 	{ $cmd = "enable $prn{'name'} ; disable -r \"$prn{'enabled_why'}\" $prn{'name'}"; }
 else { $cmd = "enable $prn{'name'} ; disable $prn{'name'}"; }
-$out = `$cmd 2>&1`;
+$out = &backquote_logged("$cmd 2>&1");
 
 # accepting or rejecting requests
 if ($prn{'accepting'}) { $cmd = "accept $prn{'name'}"; }
 elsif ($prn{'accepting_why'})
 	{ $cmd = "accept $prn{'name'} ; reject -r \"$prn{'accepting_why'}\" $prn{'name'}"; }
 else { $cmd = "accept $prn{'name'} ; reject $prn{'name'}"; }
-$out = `$cmd 2>&1`;
+$out = &backquote_logged("$cmd 2>&1");
 }
 
 # delete_printer(name)
@@ -228,7 +231,7 @@ $out = `$cmd 2>&1`;
 sub delete_printer
 {
 local($out);
-$out = `lpadmin -x $_[0] 2>&1`;
+$out = &backquote_logged("lpadmin -x $_[0] 2>&1");
 if ($?) { &error("lpadmin failed : <pre>$out</pre>"); }
 }
 
@@ -237,7 +240,7 @@ if ($?) { &error("lpadmin failed : <pre>$out</pre>"); }
 sub cancel_job
 {
 local($out);
-$out = `cancel $_[1] 2>&1`;
+$out = &backquote_logged("cancel $_[1] 2>&1");
 if ($?) { &error("cancel failed : <pre>$out</pre>"); }
 sleep(1);
 }
@@ -256,7 +259,7 @@ return -1;
 # Start lpsched
 sub start_sched
 {
-local $out = `/usr/lib/lp/lpsched 2>&1`;
+local $out = &backquote_logged("/usr/lib/lp/lpsched 2>&1");
 if ($?) { &error("failed to start lpsched : <tt>$out</tt>"); }
 }
 
@@ -264,7 +267,7 @@ if ($?) { &error("failed to start lpsched : <tt>$out</tt>"); }
 # Stop the running lpsched process
 sub stop_sched
 {
-local $out = `lpshut 2>&1`;
+local $out = &backquote_logged("lpshut 2>&1");
 if ($?) { &error("lpshut failed : <tt>$out</tt>"); }
 }
 
