@@ -185,15 +185,19 @@ else {
 	$pos = @$lref;
 	}
 if ($_[4]) {
+	local $ov = $old->{'value'};
+	$ov =~ s/\n$//;
 	local $v = $_[3];
 	$v =~ s/\n$//;
 	local @lines = split(/\n/, $v, -1);
 	@lines = map { s/^-/--/; s/^$/-/; $_ } @lines;
-	splice(@$lref, $pos, $olen, ("$_[2]        <<   END", @lines, "END"));
+	splice(@$lref, $pos, $olen, ("$_[2]        <<   END", @lines, "END"))
+		if (!$old || $v ne $ov);
 	$nlen = @lines + 2;
 	}
 else {
-	splice(@$lref, $pos, $olen, "$_[2] = $_[3]");
+	splice(@$lref, $pos, $olen, "$_[2] = $_[3]")
+		if (!$old || $_[3] ne $old->{'value'});
 	$nlen = 1;
 	}
 if ($old && $nlen != $olen) {
