@@ -19,6 +19,25 @@ if (!$@) {
 		}
 	}
 
+# check if the syslog module is available to log hacking attempts
+eval "use Sys::Syslog qw(:DEFAULT setlogsock)";
+if (!$@) {
+	$use_syslog = 1;
+	}
+
+# check if the PAM module is available to authenticate
+eval "use Authen::PAM";
+if (!$@) {
+	# check if the PAM authentication can be used by opening a handle
+	if (! ref($pamh = new Authen::PAM("webmin", "root", \&pam_conv_func))) {
+		print STDERR "PAM module available, but error during init !\n";
+		print STDERR "Disabling PAM functions.\n";
+		}
+	else {
+		$use_pam = 1;
+		}
+	}
+
 # Get miniserv's perl path and location
 $miniserv_path = $0;
 open(SOURCE, $miniserv_path);
@@ -59,7 +78,8 @@ close(CONF);
 	  "addtype_gif", "image/gif",
 	  "addtype_jpg", "image/jpeg",
 	  "addtype_jpeg", "image/jpeg",
-	  "realm", "MiniServ"
+	  "realm", "MiniServ",
+	  "session_login", "/session_login.cgi"
 	 );
 foreach $v (keys %vital) {
 	if (!$config{$v}) {
@@ -69,6 +89,12 @@ foreach $v (keys %vital) {
 		$config{$v} = $vital{$v};
 		}
 	}
+if (!$config{'sessiondb'}) {
+	$config{'pidfile'} =~ /^(.*)\/[^\/]+$/;
+	$config{'sessiondb'} = "$1/sessiondb";
+	}
+die "Session authentication cannot be used in inetd mode"
+	if ($config{'inetd'} && $config{'session'});
 
 # init days and months for http_date
 @weekday = ( "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" );
@@ -111,15 +137,22 @@ if ($use_ssl) {
 		}
 	}
 
+# Setup syslog support if possible and if requested
+if (!$config{'syslog'}) { $use_syslog = 0; }
+if ($use_syslog) {
+	openlog("webmin", "cons,pid,ndelay", "daemon");
+	}
+
 # Read MIME types file and add extra types
 if ($config{"mimetypes"} ne "") {
 	open(MIME, $config{"mimetypes"});
 	while(<MIME>) {
-		chop;
-		/^(\S+)\s+(.*)$/;
-		$type = $1; @exts = split(/\s+/, $2);
-		foreach $ext (@exts) {
-			$mime{$ext} = $type;
+		chop; s/#.*$//;
+		if (/^(\S+)\s+(.*)$/) {
+			$type = $1; @exts = split(/\s+/, $2);
+			foreach $ext (@exts) {
+				$mime{$ext} = $type;
+				}
 			}
 		}
 	close(MIME);
@@ -146,18 +179,17 @@ if ($config{'log'}) {
 
 if ($config{'inetd'}) {
 	# We are being run from inetd - go direct to handling the request
+	$SIG{'HUP'} = 'IGNORE';
 	$SIG{'TERM'} = 'DEFAULT';
 	$SIG{'PIPE'} = 'DEFAULT';
-	$SIG{'HUP'} = 'IGNORE';
 	open(SOCK, "+>&STDIN");
 
 	# Check if it is time for the logfile to be cleared
 	if ($config{'logclear'}) {
-		$write_logtime = 0;
-		if (open(LOGTIME, "$config{'logfile'}.time")) {
-			<LOGTIME> =~ /(\d+)/;
-			close(LOGTIME);
-			if ($1 && $1+$config{'logtime'}*60*60 < time()){
+		local $write_logtime = 0;
+		local @st = stat("$config{'logfile'}.time");
+		if (@st) {
+			if ($st[9]+$config{'logtime'}*60*60 < time()){
 				# need to clear log
 				$write_logtime = 1;
 				unlink($config{'logfile'});
@@ -208,8 +240,11 @@ socket(MAIN, PF_INET, SOCK_STREAM, $proto) ||
 	die "Failed to open main socket : $!";
 setsockopt(MAIN, SOL_SOCKET, SO_REUSEADDR, pack("l", 1));
 $baddr = $config{"bind"} ? inet_aton($config{"bind"}) : INADDR_ANY;
-bind(MAIN, sockaddr_in($config{port}, $baddr)) ||
-	die "Failed to bind port $config{port} : $!";
+for($i=0; $i<5; $i++) {
+	last if (bind(MAIN, sockaddr_in($config{port}, $baddr)));
+	sleep(1);
+	}
+die "Failed to bind port $config{port} : $!" if ($i == 5);
 listen(MAIN, SOMAXCONN);
 
 if ($config{'listen'}) {
@@ -240,11 +275,10 @@ close(PIDFILE);
 if ($config{'logclear'}) {
 	if (!($logclearer = fork())) {
 		while(1) {
-			$write_logtime = 0;
-			if (open(LOGTIME, "$config{'logfile'}.time")) {
-				<LOGTIME> =~ /(\d+)/;
-				close(LOGTIME);
-				if ($1 && $1+$config{'logtime'}*60*60 < time()){
+			local $write_logtime = 0;
+			local @st = stat("$config{'logfile'}.time");
+			if (@st) {
+				if ($st[9]+$config{'logtime'}*60*60 < time()){
 					# need to clear log
 					$write_logtime = 1;
 					unlink($config{'logfile'});
@@ -263,6 +297,11 @@ if ($config{'logclear'}) {
 	push(@childpids, $logclearer);
 	}
 
+# Setup the logout time dbm if needed
+if ($config{'session'}) {
+	dbmopen(%sessiondb, $config{'sessiondb'}, 0700);
+	}
+
 # Run the main loop
 $SIG{'HUP'} = 'miniserv::trigger_restart';
 $SIG{'TERM'} = 'miniserv::term_handler';
@@ -274,7 +313,7 @@ while(1) {
 	# wait for a new connection, or a message from a child process
 	undef($rmask);
 	vec($rmask, fileno(MAIN), 1) = 1;
-	if ($config{'passdelay'}) {
+	if ($config{'passdelay'} || $config{'session'}) {
 		for($i=0; $i<@passin; $i++) {
 			vec($rmask, fileno($passin[$i]), 1) = 1;
 			}
@@ -283,6 +322,7 @@ while(1) {
 
 	local $sel = select($rmask, undef, undef, 10);
 	if ($need_restart) { &restart_miniserv(); }
+	local $time_now = time();
 
 	# Clean up finished processes
 	local($pid);
@@ -290,38 +330,46 @@ while(1) {
 		@childpids = grep { $_ != $pid } @childpids;
 		} while($pid > 0);
 
+	# run the unblocking procedure to check if enough time has passed to
+	# unblock hosts that heve been blocked because of password failures
+	if ($config{'blockhost_failures'}) {
+		$i = 0;
+		while ($i <= $#deny) {
+			if ($blockhosttime{$deny[$i]} && $config{'blockhost_time'} != 0 &&
+			    ($time_now - $blockhosttime{$deny[$i]}) >= $config{'blockhost_time'}) {
+				# the host can be unblocked now
+				$hostfail{$deny[$i]} = 0;
+				splice(@deny, $i, 1);
+				}
+			$i++;
+			}
+		}
+
+	if ($config{'session'}) {
+		# Remove sessions with more than 24 hours inactivity
+		foreach $s (keys %sessiondb) {
+			local ($user, $ltime) = split(/\s+/, $sessiondb{$s});
+			if ($time_now - $ltime > 24*60*60) {
+				delete($sessiondb{$s});
+				}
+			}
+		}
+
 	next if ($sel <= 0);
 	if (vec($rmask, fileno(MAIN), 1)) {
 		# got new connection
 		$acptaddr = accept(SOCK, MAIN);
-
-		# run the unblocking procedure to check if enough time has passed to
-		# unblock hosts that heve been blocked because of password failures
-		if ($config{'blockhost_failures'}) {
-			$i = 0;
-			while ($i <= $#deny) {
-				if ($blockhosttime{$deny[$i]} && $config{'blockhost_time'} != 0 &&
-						(time() - $blockhosttime{$deny[$i]}) >= $config{'blockhost_time'}) {
-					# the host can be unblocked now
-					#print STDERR "unblocking $deny[$i]\n";
-					$hostfail{$deny[$i]} = 0;
-					splice(@deny, $i, 1);
-					}
-				$i++;
-				}
-			}
-		
 		if (!$acptaddr) { next; }
 
 		# create pipes
-		if ($config{'passdelay'}) {
+		if ($config{'passdelay'} || $config{'session'}) {
 			$PASSINr = "PASSINr$p"; $PASSINw = "PASSINw$p";
 			$PASSOUTr = "PASSOUTr$p"; $PASSOUTw = "PASSOUTw$p";
 			$p++;
 			pipe($PASSINr, $PASSINw);
 			pipe($PASSOUTr, $PASSOUTw);
-			select($PASSINw); $| = 1;
-			select($PASSOUTw); $| = 1;
+			select($PASSINw); $| = 1; select($PASSINr); $| = 1;
+			select($PASSOUTw); $| = 1; select($PASSOUTw); $| = 1;
 			}
 		select(STDOUT);
 
@@ -347,7 +395,7 @@ while(1) {
 				}
 
 			# close useless pipes
-			if ($config{'passdelay'}) {
+			if ($config{'passdelay'} || $config{'session'}) {
 				foreach $p (@passin) { close($p); }
 				foreach $p (@passout) { close($p); }
 				close($PASSINr); close($PASSOUTw);
@@ -373,7 +421,7 @@ while(1) {
 			exit;
 			}
 		push(@childpids, $handpid);
-		if ($config{'passdelay'}) {
+		if ($config{'passdelay'} || $config{'session'}) {
 			close($PASSINw); close($PASSOUTr);
 			push(@passin, $PASSINr); push(@passout, $PASSOUTw);
 			}
@@ -398,16 +446,16 @@ while(1) {
 		if (vec($rmask, fileno($passin[$i]), 1)) {
 			# this sub-process is asking about a password
 			$infd = $passin[$i]; $outfd = $passout[$i];
-			if (<$infd> =~ /^(\S+)\s+(\S+)\s+(\d+)/) {
+			$inline = <$infd>;
+			if ($inline =~ /^delay\s+(\S+)\s+(\S+)\s+(\d+)/) {
 				# Got a delay request from a subprocess.. for
 				# valid logins, there is no delay (to prevent
 				# denial of service attacks), but for invalid
 				# logins the delay increases with each failed
 				# attempt.
-				#print STDERR "got $1 $2 $3\n";
 				if ($3) {
 					# login OK.. no delay
-					print $outfd "0\n";
+					print $outfd "0 0\n";
 					$hostfail{$2} = 0;
 					}
 				else {
@@ -417,19 +465,53 @@ while(1) {
  					if ($config{'blockhost_failures'} &&
 					    $hostfail{$2} >= $config{'blockhost_failures'}) {
 						push(@deny, $2);					
-						$blockhosttime{$2} = time();
+						$blockhosttime{$2} = $time_now;
 						$blocked = 1;
+						if ($use_syslog) {
+							local $logtext = "Security alert: Host $2 ".
+							  "blocked after $config{'blockhost_failures'} ".
+							  "failed logins for user $1";
+							syslog("crit", $logtext);
+							}
 						}
 					else {
 						$blocked = 0;
 						}
 					$dl = $userdlay{$1} -
-					      int((time() - $userlast{$1})/50);
+					      int(($time_now - $userlast{$1})/50);
 					$dl = $dl < 0 ? 0 : $dl+1;
 					print $outfd "$dl $blocked\n";
 					$userdlay{$1} = $dl;
 					}
-				$userlast{$1} = time();
+				$userlast{$1} = $time_now;
+				}
+			elsif ($inline =~ /^verify\s+(\S+)/) {
+				# Verifying a session ID
+				local $session_id = $1;
+				if (!defined($sessiondb{$session_id})) {
+					print $outfd "0 0\n";
+					}
+				else {
+					local ($user, $ltime) = split(/\s+/, $sessiondb{$session_id});
+					if ($config{'logouttime'} &&
+					    $time_now - $ltime > $config{'logouttime'}*60) {
+						print $outfd "1 ",$time_now - $ltime,"\n";
+						delete($sessiondb{$session_id});
+						}
+					else {
+						print $outfd "2 $user\n";
+						$sessiondb{$session_id} = "$user $time_now";
+						}
+					}
+				}
+			elsif ($inline =~ /^new\s+(\S+)\s+(\S+)/) {
+				# Creating a new session
+				$sessiondb{$1} = "$2 $time_now";
+				}
+			elsif ($inline =~ /^delete\s+(\S+)/) {
+				# Logging out a session
+				print $outfd $sessiondb{$1} ? 1 : 0,"\n";
+				delete($sessiondb{$1});
 				}
 			else {
 				# close pipe
@@ -447,6 +529,13 @@ while(1) {
 sub handle_request
 {
 $acptip = inet_ntoa((unpack_sockaddr_in($_[0]))[1]);
+if ($config{'loghost'}) {
+	$acpthost = gethostbyaddr(inet_aton($acptip), AF_INET);
+	$acpthost = $acptip if (!$acpthost);
+	}
+else {
+	$acpthost = $acptip;
+	}
 $datestr = &http_date(time());
 
 # Read the HTTP request and headers
@@ -466,6 +555,7 @@ if (defined($header{'host'})) {
 	if ($header{'host'} =~ /^([^:]+):([0-9]+)$/) { $host = $1; $port = $2; }
 	else { $host = $header{'host'}; }
 	}
+undef(%in);
 if ($page =~ /^([^\?]+)\?(.*)$/) {
 	# There is some query string information
 	$page = $1;
@@ -476,6 +566,16 @@ if ($page =~ /^([^\?]+)\?(.*)$/) {
     		$queryargs =~ s/%(..)/pack("c",hex($1))/ge;
 		$querystring = "";
 		}
+	else {
+		# Parse query-string parameters
+		local @in = split(/\&/, $querystring);
+		foreach $i (@in) {
+			local ($k, $v) = split(/=/, $i, 2);
+			$k =~ s/\+/ /g; $k =~ s/%(..)/pack("c",hex($1))/ge;
+			$v =~ s/\+/ /g; $v =~ s/%(..)/pack("c",hex($1))/ge;
+			$in{$k} = $v;
+			}
+		}
 	}
 
 # replace %XX sequences in page
@@ -487,34 +587,21 @@ if (@deny && &ip_match($acptip, @deny) ||
 	return 0;
 	}
 
-# check for the logout flag file, and if existant deny authentication once
-if ($config{'logout'} && -r $config{'logout'}) {
-	&write_data("HTTP/1.0 401 Unauthorized\r\n");
-	&write_data("Server: $config{server}\r\n");
-	&write_data("Date: $datestr\r\n");
-	&write_data("WWW-authenticate: Basic ".
-		    "realm=\"$config{realm}\"\r\n");
-	&write_data("Content-type: text/html\r\n");
-	&write_keep_alive(0);
-	&write_data("\r\n");
-	&reset_byte_count();
-	&write_data("<title>Please Login</title>\n");
-	&write_data("<h1>Please Login</h1>\n");
-	&write_data("Please login to the server as a new user.<p>\n");
-	&log_request($acptip, undef, $reqline, 401, &byte_count());
-	open(LOGOUT, $config{'logout'});
+# check for the logout flag file, and if existant deny authentication
+if ($config{'logout'} && -r $config{'logout'}.$in{'miniserv_logout_id'}) {
+	$deny_authentication++;
+	open(LOGOUT, $config{'logout'}.$in{'miniserv_logout_id'});
 	chop($count = <LOGOUT>);
 	close(LOGOUT);
 	$count--;
 	if ($count > 0) {
-		open(LOGOUT, ">$config{'logout'}");
+		open(LOGOUT, ">$config{'logout'}$in{'miniserv_logout_id'}");
 		print LOGOUT "$count\n";
 		close(LOGOUT);
 		}
 	else {
-		unlink($config{'logout'});
+		unlink($config{'logout'}.$in{'miniserv_logout_id'});
 		}
-	return 0;
 	}
 
 # Check for password if needed
@@ -538,50 +625,139 @@ if (%users) {
 		}
 
 	# Check for normal HTTP authentication
-	if (!$validated && $header{authorization} =~ /^basic\s+(\S+)$/i) {
+	if (!$validated && !$deny_authentication && !$config{'session'} &&
+	    $header{authorization} =~ /^basic\s+(\S+)$/i) {
 		# authorization given..
 		($authuser, $authpass) = split(/:/, &b64decode($1));
-		if ($authuser && $users{$authuser} && $users{$authuser} eq
-		    crypt($authpass, $users{$authuser})) {
-			$validated = 1;
-			}
-		#print STDERR "checking $authuser $authpass -> $validated\n";
+		$validated = &validate_user($authuser, $authpass);
 
 		if ($config{'passdelay'} && !$config{'inetd'}) {
 			# check with main process for delay
-			print $PASSINw "$authuser $acptip $validated\n";
+			print $PASSINw "delay $authuser $acptip $validated\n";
 			<$PASSOUTr> =~ /(\d+) (\d+)/;
-			#print STDERR "sleeping for $1\n";
 			$blocked = $2;
 			sleep($1);
 			}
 		}
+
+	# Check for new session validation
+	if ($config{'session'} && !$deny_authentication && $page eq $config{'session_login'}) {
+		local $ok = &validate_user($in{'user'}, $in{'pass'});
+
+		# check with main process for delay
+		if ($config{'passdelay'} && $in{'user'}) {
+			print $PASSINw "delay $in{'user'} $acptip $ok\n";
+			<$PASSOUTr> =~ /(\d+) (\d+)/;
+			$blocked = $2;
+			sleep($1);
+			}
+
+		if ($ok) {
+			# Logged in OK! Tell the main process about the new SID
+			local $sid = time();
+			local $mul = 1;
+			foreach $c (split(//, crypt($in{'pass'}, substr($$, -2)))) {
+				$sid += ord($c) * $mul;
+				$mul *= 3;
+				}
+			print $PASSINw "new $sid $in{'user'}\n";
+
+			# Set cookie and redirect
+			&write_data("HTTP/1.0 302 Moved Temporarily\r\n");
+			&write_data("Date: $datestr\r\n");
+			&write_data("Server: $config{'server'}\r\n");
+			$portstr = $port == 80 && !$use_ssl ? "" :
+				   $port == 443 && $use_ssl ? "" : ":$port";
+			$prot = $use_ssl ? "https" : "http";
+			&write_data("Set-Cookie: sid=$sid; path=/\r\n");
+			&write_data("Location: $prot://$host$portstr$in{'page'}\r\n");
+			&write_keep_alive(0);
+			&write_data("\r\n");
+			&log_request($acpthost, $authuser, $reqline, 302, 0);
+			return 0;
+			}
+		elsif ($in{'logout'} && $header{'cookie'} =~ /sid=(\d+)/) {
+			# Logout clicked .. remove the session
+			print $PASSINw "delete $1\n";
+			local $dummy = <$PASSINr>;
+			$logout = 1;
+			$already_session_id = undef;
+			}
+		else {
+			# Login failed .. display the form again
+			$failed_user = $in{'user'};
+			$page = $in{'page'};
+			$already_session_id = undef;
+			}
+		}
+
+	# Check for an existing session
+	if ($config{'session'}) {
+		if ($already_session_id) {
+			$session_id = $already_session_id;
+			$authuser = $already_authuser;
+			$validated = 1;
+			}
+		elsif (!$deny_authentication && $header{'cookie'} =~ /sid=(\d+)/) {
+			$session_id = $1;
+			print $PASSINw "verify $session_id\n";
+			<$PASSOUTr> =~ /(\d+)\s+(\S+)/;
+			if ($1 == 2) {
+				# Valid session continuation
+				$validated = 1;
+				$authuser = $2;
+				$already_session_id = $session_id;
+				$already_authuser = $authuser;
+				}
+			elsif ($1 == 1) {
+				# Session timed out
+				$timed_out = $2;
+				}
+			else {
+				# Invalid session ID .. don't set verified
+				}
+			}
+		}
+
 	if (!$validated) {
 		if ($blocked == 0) {
 			# No password given.. ask
-			&write_data("HTTP/1.0 401 Unauthorized\r\n");
-			&write_data("Date: $datestr\r\n");
-			&write_data("Server: $config{'server'}\r\n");
-			&write_data("WWW-authenticate: Basic ".
-				   "realm=\"$config{'realm'}\"\r\n");
-			&write_keep_alive(0);
-			&write_data("Content-type: text/html\r\n");
-			&write_data("\r\n");
-			&reset_byte_count();
-			&write_data("<html>\n");
-			&write_data("<head><title>Unauthorized</title></head>\n");
-			&write_data("<body><h1>Unauthorized</h1>\n");
-			&write_data("A password is required to access this\n");
-			&write_data("web server. Please try again. <p>\n");
-			&write_data("</body></html>\n");
-			&log_request($acptip, undef, $reqline, 401, &byte_count());
+			if ($config{'session'}) {
+				# Force CGI for session login
+				$validated = 1;
+				$querystring = "page=".&urlize($page);
+				$querystring .= "&failed=$failed_user" if ($failed_user);
+				$querystring .= "&timed_out=$timed_out" if ($timed_out);
+				$querystring .= "&logout=1" if ($logout);
+				$queryargs = "";
+				$page = $config{'session_login'};
+				}
+			else {
+				# Ask for login with HTTP authentication
+				&write_data("HTTP/1.0 401 Unauthorized\r\n");
+				&write_data("Date: $datestr\r\n");
+				&write_data("Server: $config{'server'}\r\n");
+				&write_data("WWW-authenticate: Basic ".
+					   "realm=\"$config{'realm'}\"\r\n");
+				&write_keep_alive(0);
+				&write_data("Content-type: text/html\r\n");
+				&write_data("\r\n");
+				&reset_byte_count();
+				&write_data("<html>\n");
+				&write_data("<head><title>Unauthorized</title></head>\n");
+				&write_data("<body><h1>Unauthorized</h1>\n");
+				&write_data("A password is required to access this\n");
+				&write_data("web server. Please try again. <p>\n");
+				&write_data("</body></html>\n");
+				&log_request($acpthost, undef, $reqline, 401, &byte_count());
+				return 0;
+				}
 			}
 		else {
 			# when the host has been blocked, give it an error message
 			&http_error(403, "Access denied for $acptip. The host has been blocked "
 				."because of too many authentication failures.");
 			}
-		return 0;
 		}
 
 	# Check per-user IP access control
@@ -641,7 +817,7 @@ if (-d $full) {
 		&write_data("Location: $prot://$host$portstr$page/\r\n");
 		&write_keep_alive(0);
 		&write_data("\r\n");
-		&log_request($acptip, $authuser, $reqline, 302, 0);
+		&log_request($acpthost, $authuser, $reqline, 302, 0);
 		return 0;
 		}
 	# A directory.. check for index files
@@ -683,7 +859,7 @@ if (-d $full) {
 		 $df, $df, $fdate, $stbuf[7]);
 		}
 	closedir(DIR);
-	&log_request($acptip, $authuser, $reqline, 200, &byte_count());
+	&log_request($acpthost, $authuser, $reqline, 200, &byte_count());
 	return 0;
 	}
 
@@ -704,7 +880,7 @@ if (&get_type($full) eq "internal/cgi") {
 	$ENV{"SERVER_ADMIN"} = $config{"email"};
 	$ENV{"SERVER_ROOT"} = $config{"root"};
 	$ENV{"SERVER_PORT"} = $port;
-	$ENV{"REMOTE_HOST"} = $acptip;
+	$ENV{"REMOTE_HOST"} = $acpthost;
 	$ENV{"REMOTE_ADDR"} = $acptip;
 	$ENV{"REMOTE_USER"} = $authuser if (defined($authuser));
 	$ENV{"SSL_USER"} = $peername if ($validated == 2);
@@ -719,6 +895,7 @@ if (&get_type($full) eq "internal/cgi") {
 	$ENV{"QUERY_STRING"} = $querystring;
 	$ENV{"MINISERV_CONFIG"} = $conf;
 	$ENV{"HTTPS"} = "ON" if ($use_ssl);
+	$ENV{"SESSION_ID"} = $session_id if ($session_id);
 	if (defined($header{"content-length"})) {
 		$ENV{"CONTENT_LENGTH"} = $header{"content-length"};
 		}
@@ -851,7 +1028,7 @@ if (&get_type($full) eq "internal/cgi") {
 		else {
 			&write_data("HTTP/1.0 200 Document follows\r\n");
 			&write_data("Date: $datestr\r\n");
-			&write_data("Server: $config{server}\r\n");
+			&write_data("Server: $config{'server'}\r\n");
 			&write_keep_alive(0);
 			}
 		foreach $h (keys %cgiheader) {
@@ -885,7 +1062,7 @@ else {
 	}
 
 # log the request
-&log_request($acptip, $authuser, $reqline,
+&log_request($acpthost, $authuser, $reqline,
 	     $cgiheader{"location"} ? "302" : "200", &byte_count());
 return $rv;
 }
@@ -905,7 +1082,7 @@ close(CGIOUT);
 if ($_[2]) {
 	&write_data("<pre>$_[2]</pre>\n");
 	}
-&log_request($acptip, $authuser, $reqline, $_[0], &byte_count());
+&log_request($acpthost, $authuser, $reqline, $_[0], &byte_count());
 exit if (!$_[3]);
 }
 
@@ -1092,11 +1269,11 @@ sub reset_byte_count { $write_data_count = 0; }
 # byte_count()
 sub byte_count { return $write_data_count; }
 
-# log_request(address, user, request, code, bytes)
+# log_request(hostname, user, request, code, bytes)
 sub log_request
 {
 if ($config{'log'}) {
-	local(@tm, $dstr, $addr, $user, $ident);
+	local(@tm, $dstr, $user, $ident);
 	if ($config{'logident'}) {
 		# add support for rfc1413 identity checking here
 		}
@@ -1105,8 +1282,6 @@ if ($config{'log'}) {
 	$dstr = sprintf "%2.2d/%s/%4.4d:%2.2d:%2.2d:%2.2d %s",
 			$tm[3], $make_date_marr[$tm[4]], $tm[5]+1900,
 	                $tm[2], $tm[1], $tm[0], $timezone;
-	$addr = $config{'loghost'} ? gethostbyaddr(inet_aton($_[0]), AF_INET)
-				   : $_[0];
 	$user = $_[1] ? $_[1] : "-";
 	if (fileno(MINISERVLOG)) {
 		seek(MINISERVLOG, 0, 2);
@@ -1115,7 +1290,7 @@ if ($config{'log'}) {
 		open(MINISERVLOG, ">>$config{'logfile'}");
 		chmod(0600, $config{'logfile'});
 		}
-	print MINISERVLOG "$addr $ident $user [$dstr] \"$_[2]\" $_[3] $_[4]\n";
+	print MINISERVLOG "$_[0] $ident $user [$dstr] \"$_[2]\" $_[3] $_[4]\n";
 	close(MINISERVLOG);
 	}
 }
@@ -1286,8 +1461,63 @@ if ($doing_eval) {
 	# A CGI program called exit! This is a horrible hack to 
 	# finish up before really exiting
 	close(SOCK);
-	&log_request($acptip, $authuser, $reqline,
+	&log_request($acpthost, $authuser, $reqline,
 		     $cgiheader{"location"} ? "302" : "200", &byte_count());
 	}
 }
 
+# urlize
+# Convert a string to a form ok for putting in a URL
+sub urlize {
+  local($tmp, $tmp2, $c);
+  $tmp = $_[0];
+  $tmp2 = "";
+  while(($c = chop($tmp)) ne "") {
+	if ($c !~ /[A-z0-9]/) {
+		$c = sprintf("%%%2.2X", ord($c));
+		}
+	$tmp2 = $c . $tmp2;
+	}
+  return $tmp2;
+}
+
+# validate_user(username, password)
+sub validate_user
+{
+return 0 if (!$_[0] || !$users{$_[0]});
+if ($users{$_[0]} eq 'x' && $use_pam) {
+	local $pamh = new Authen::PAM("webmin", $pam_username, \&pam_conv_func);
+	if (!ref($pamh)) {
+		print STDERR "PAM init failed : $pamh\n";
+		return 0;
+		}
+	$pam_username = $_[0];
+	$pam_password = $_[1];
+	local $pam_ret = $pamh->pam_authenticate();
+	print STDERR "pam_ret = $pam_ret\n";
+	return $pam_ret == PAM_SUCCESS ? 1 : 0;
+	}
+else {
+	return $users{$_[0]} eq crypt($_[1], $users{$_[0]}) ? 1 : 0;
+	}
+}
+
+# the PAM conversation function for interactive logins
+sub pam_conv_func
+{
+my @res;
+while ( @_ ) {
+	my $code = shift;
+	my $msg = shift;
+	my $ans = "";
+
+	$ans = $pam_username if ($code == PAM_PROMPT_ECHO_ON() );
+	$ans = $pam_password if ($code == PAM_PROMPT_ECHO_OFF() );
+
+	push @res, PAM_SUCCESS();
+	push @res, $ans;
+	}
+push @res, PAM_SUCCESS();
+return @res;
+}
+
