@@ -180,12 +180,12 @@ if ($_[0] eq "swap") {
         if (defined($options{"pri"})) {
 		$opts = "-p $options{'pri'}";
 		}
-	$out = `swapon $opts $_[1] 2>&1`;
+	$out = &backquote_logged("swapon $opts $_[1] 2>&1");
 	if ($? && !($out =~ /already enabled for paging/)) { return $out; }
 	}
 else {
 	$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
-	$out = `mount -F $_[2] $opts -- $_[1] $_[0] 2>&1`;
+	$out = &backquote_logged("mount -F $_[2] $opts -- $_[1] $_[0] 2>&1");
 	if ($?) { return $out; }
 	}
 return 0;
@@ -202,7 +202,7 @@ if ($_[0] eq "swap") {
         &error("Swap space cannot be removed");
         }
 else {
-	$out = `umount $_[0] 2>&1`;
+	$out = &backquote_logged("umount $_[0] 2>&1");
 	}
 if ($?) { return $out; }
 return 0;
@@ -1177,4 +1177,9 @@ sub device_name
 return $_[0];
 }
 
+sub files_to_lock
+{
+return ( $config{'fstab_file'} );
+}
+
 1;
