@@ -227,7 +227,7 @@ elsif ($_[2] eq "rumba") {
 	$dir = $_[0];
 	if (`fuser -c $_[0] 2>/dev/null` =~ /\d/) { return "$_[0] is busy"; }
 	if (`cat /etc/mnttab` =~ /rumba-(\d+)\s+$dir\s+nfs/) {
-		kill('TERM', $1) || return "Failed to kill rumba";
+		&kill_logged('TERM', $1) || return "Failed to kill rumba";
 		}
 	else { return "Failed to find rumba pid"; }
 	sleep(1);
@@ -1460,4 +1460,9 @@ sub device_name
 return $_[0];
 }
 
+sub files_to_lock
+{
+return ( $config{'fstab_file'} );
+}
+
 1;
