@@ -9,6 +9,9 @@ $autofs_support = &has_command("automount");
 if (&has_command("mount.smb")) {
 	$smbfs_support = `mount.smb -v` =~ /username=/i ? 4 : 3;
 	}
+elsif (&has_command("mount.smbfs")) {
+	$smbfs_support = `mount.smbfs -v` =~ /username=/i ? 4 : 3;
+	}
 elsif (&has_command("smbmount")) {
 	$smbfs_support = `smbmount -v` =~ /Version\s+2/i ? 2 : 1;
 	}
@@ -396,18 +399,18 @@ sub mount_dir
 local($out, $opts, $shar, %options, %smbopts);
 if ($_[2] eq "swap") {
 	# Use swapon to add the swap space..
-	$out = `swapon $_[1] 2>&1`;
+	$out = &backquote_logged("swapon $_[1] 2>&1");
 	if ($out =~ /Invalid argument/) {
 		# looks like this swap partition isn't ready yet.. set it up
-		$out = `mkswap $_[1] 2>&1`;
+		$out = &backquote_logged("mkswap $_[1] 2>&1");
 		if ($?) { return "mkswap failed : <pre>$out</pre>"; }
-		$out = `swapon $_[1] 2>&1`;
+		$out = &backquote_logged("swapon $_[1] 2>&1");
 		}
 	if ($?) { return "<pre>$out</pre>"; }
 	}
 elsif ($_[2] eq "auto") {
 	# Old automounter filesystem
-	$out = `amd $_[0] $_[1] >/dev/null 2>/dev/null`;
+	$out = &backquote_logged("amd $_[0] $_[1] >/dev/null 2>/dev/null");
 	if ($?) { return $text{'linux_eamd'}; }
 	}
 elsif ($_[2] eq "autofs") {
@@ -415,7 +418,7 @@ elsif ($_[2] eq "autofs") {
 	$opts = &autofs_args($_[3]);
 	$type = $_[1] !~ /^\// ? "yp" :
 		(-x $_[1]) ? "program" : "file";
-	$out = `automount $opts $_[0] $type $_[1] 2>&1`;
+	$out = &backquote_logged("automount $opts $_[0] $type $_[1] 2>&1");
 	if ($?) { return &text('linux_eauto', "<pre>$out</pre>"); }
 	}
 elsif ($_[2] eq "smbfs") {
@@ -424,10 +427,10 @@ elsif ($_[2] eq "smbfs") {
 	if ($smbfs_support >= 3) {
 		# SMB filesystem mounted with mount command
 		$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
-		$out = `mount -t $_[2] $opts $shar $_[0] 2>&1`;
+		$out = &backquote_logged("mount -t $_[2] $opts $shar $_[0] 2>&1");
 		if ($? || $out =~ /failed/i) {
-			system("umount $_[0] >/dev/null 2>&1");
-			return "<pre>$out</pre>";
+			&system_logged("umount $_[0] >/dev/null 2>&1");
+			return "mount -t $_[2] $opts $shar $_[0] : <pre>$out</pre>";
 			}
 		}
 	elsif ($smbfs_support == 2) {
@@ -441,12 +444,12 @@ elsif ($_[2] eq "smbfs") {
 		    ($options{'machinename'} ? "-I $options{'machinename'} " : "");
 		&foreign_require("proc", "proc-lib.pl");
 		local ($fh, $fpid) = &foreign_call(
-			"proc", "pty_process_exec", "sh", "-c",
+			"proc", "pty_process_exec_logged", "sh", "-c",
 			"smbmount $shar $_[0] -d 0 $opts");
 		if ($options{'passwd'}) {
 			local $w = &wait_for($fh, "word:");
 			if ($w < 0) {
-				system("umount $_[0] >/dev/null 2>&1");
+				&system_logged("umount $_[0] >/dev/null 2>&1");
 				return $text{'linux_esmbconn'};
 				}
 			local $p = "$options{'passwd'}\n";
@@ -457,7 +460,7 @@ elsif ($_[2] eq "smbfs") {
 			$got .= $_;
 			}
 		if ($got =~ /failed/) {
-			system("umount $_[0] >/dev/null 2>&1");
+			&system_logged("umount $_[0] >/dev/null 2>&1");
 			return "<pre>$got</pre>\n";
 			}
 		close($fh);
@@ -478,9 +481,9 @@ elsif ($_[2] eq "smbfs") {
 		   ($options{gid} ? "-g $options{gid} " : "").
 		   ($options{fmode} ? "-f $options{fmode} " : "").
 		   ($options{dmode} ? "-d $options{dmode} " : "");
-		$out = `smbmount $shar $_[0] $opts 2>&1`;
+		$out = &backquote_logged("smbmount $shar $_[0] $opts 2>&1");
 		if ($out) {
-			system("umount $_[0] >/dev/null 2>&1");
+			&system_logged("umount $_[0] >/dev/null 2>&1");
 			return "<pre>$out</pre>";
 			}
 		}
@@ -491,7 +494,7 @@ elsif ($_[2] eq "smbfs") {
 else {
 	# some filesystem supported by mount
 	$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
-	$out = `mount -t $_[2] $opts $_[1] $_[0] 2>&1`;
+	$out = &backquote_logged("mount -t $_[2] $opts $_[1] $_[0] 2>&1");
 	if ($?) { return "<pre>$out</pre>"; }
 	}
 return 0;
@@ -506,14 +509,14 @@ sub unmount_dir
 local($out, %smbopts, $dir);
 if ($_[2] eq "swap") {
 	# Use swapoff to remove the swap space..
-	$out = `swapoff $_[1]`;
+	$out = &backquote_logged("swapoff $_[1]");
 	if ($?) { return "<pre>$out</pre>"; }
 	}
 elsif ($_[2] eq "auto") {
 	# Kill the amd process
 	$dir = $_[0];
 	if (`cat /etc/mtab` =~ /:\(pid([0-9]+)\)\s+$dir\s+(auto|nfs)\s+/) {
-		kill('TERM', $1) || return $text{'linux_ekillamd'};
+		&kill_logged('TERM', $1) || return $text{'linux_ekillamd'};
 		}
 	sleep(2);
 	}
@@ -521,11 +524,11 @@ elsif ($_[2] eq "autofs") {
 	# Kill the automount process
 	$dir = $_[0];
 	`cat /etc/mtab` =~ /automount\(pid([0-9]+)\)\s+$dir\s+autofs\s+/;
-	kill('TERM', $1) || return $text{'linux_ekillauto'};
+	&kill_logged('TERM', $1) || return $text{'linux_ekillauto'};
 	sleep(2);
 	}
 else {
-	$out = `umount $_[0] 2>&1`;
+	$out = &backquote_logged("umount $_[0] 2>&1");
 	if ($?) { return "<pre>$out</pre>"; }
 	if ($_[2] eq "smbfs") {
 		# remove options from list
@@ -1627,7 +1630,7 @@ local (@rv, $str, $lnum);
 open(AMD, $config{'auto_file'});
 while(<AMD>) {
 	s/\r|\n//g;
-	s/#*$//g;
+	s/#.*$//g;
 	if (/\[\s*(\S+)\s*\]/) {
 		$str = { 'dir' => $1,
 			 'line' => $lnum,
@@ -1668,5 +1671,10 @@ return $_[0] =~ /^\/dev\/(s|h)d([a-z])(\d+)$/ ?
 	$_[0];
 }
 
+sub files_to_lock
+{
+return ( $config{'fstab_file'}, $config{'autofs_file'}, $config{'auto_file'} );
+}
+
 1;
 
