@@ -150,14 +150,14 @@ sub mount_dir
 local($out, $opts, $shar, %options, %smbopts);
 if ($_[2] eq "swap") {
 	# Use swapon to add the swap space..
-	$out = `swapon $_[1] 2>&1`;
+	$out = &backquote_logged("swapon $_[1] 2>&1");
 	if ($?) { return "<pre>$out</pre>"; }
 	}
 else {
 	# some disk-based filesystem
 	$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
 	$opts = join(',', grep { !/quota/ } split(/,/, $opts));
-	$out = `mount -t $_[2] $opts $_[1] $_[0] 2>&1`;
+	$out = &backquote_logged("mount -t $_[2] $opts $_[1] $_[0] 2>&1");
 	if ($?) { return "<pre>$out</pre>"; }
 	}
 return 0;
@@ -175,7 +175,7 @@ if ($_[2] eq "swap") {
 	&error("Swap space cannot be removed");
 	}
 else {
-	$out = `umount $_[0] 2>&1`;
+	$out = &backquote_logged("umount $_[0] 2>&1");
 	if ($?) { return "<pre>$out</pre>"; }
 	}
 return 0;
@@ -751,5 +751,10 @@ sub device_name
 return $_[0];
 }
 
+sub files_to_lock
+{
+return ( $config{'fstab_file'} );
+}
+
 1;
 
