@@ -217,7 +217,7 @@ sub mount_dir
 local($out, $opts);
 if ($_[0] eq "swap") {
 	# Adding a swap device
-	$out = `swap -a $_[1] 2>&1`;
+	$out = &backquote_logged("swap -a $_[1] 2>&1");
 	if ($?) { return $out; }
 	}
 else {
@@ -233,7 +233,7 @@ else {
 				return &text('solaris_ecacheexists',
 					     $options{'cachedir'});
 				}
-			$out = `cfsadmin -c $options{cachedir} 2>&1`;
+			$out = &backquote_logged("cfsadmin -c $options{cachedir} 2>&1");
 			if ($?) { return $out; }
 			}
 		}
@@ -261,13 +261,13 @@ else {
 		 (defined($options{'readwrite'}) ? "-S " : "").
 		 (defined($options{'readonly'}) ? "-w " : "").
 		 (defined($options{'attr'}) ? "-e " : "");
-		$rv = system("$smbmount \"$shar\" $_[0] $opts >/tmp/$$.rumba 2>&1 </dev/null");
+		$rv = &system_logged("$smbmount \"$shar\" $_[0] $opts >/tmp/$$.rumba 2>&1 </dev/null");
 		$out = `cat /tmp/$$.rumba`; unlink("/tmp/$$.rumba");
 		if ($rv) { return "<pre>$out</pre> : rumba \"$shar\" $_[0] $opts"; }
 		}
 	else {
 		$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
-		$out = `mount -F $_[2] $opts -- $_[1] $_[0] 2>&1`;
+		$out = &backquote_logged("mount -F $_[2] $opts -- $_[1] $_[0] 2>&1");
 		if ($?) { return $out; }
 		}
 	}
@@ -281,20 +281,21 @@ return 0;
 sub unmount_dir
 {
 if ($_[0] eq "swap") {
-	$out = `swap -d $_[1] 2>&1`;
+	$out = &backquote_logged("swap -d $_[1] 2>&1");
 	}
 elsif ($_[2] eq "rumba") {
 	# kill the process (if nobody is in the directory)
 	$dir = $_[0];
-	if (`fuser -c $_[0] 2>/dev/null` =~ /\d/) { return &text('solaris_ebusy', $_[0]); }
+	if (`fuser -c $_[0] 2>/dev/null` =~ /\d/)
+		{ return &text('solaris_ebusy', $_[0]); }
 	if (`cat /etc/mnttab` =~ /(rumba|shlight)-(\d+)\s+$dir\s+nfs/) {
-		kill('TERM', $2) || return $text{'solaris_ekill'};
+		&kill_logged('TERM', $2) || return $text{'solaris_ekill'};
 		}
 	else { return $text{'solaris_epid'}; }
 	sleep(1);
 	}
 else {
-	$out = `umount $_[0] 2>&1`;
+	$out = &backquote_logged("umount $_[0] 2>&1");
 	}
 if ($?) { return $out; }
 return 0;
@@ -324,6 +325,7 @@ sub list_fstypes
 local @fs = ("ufs", "nfs", "hsfs", "pcfs", "lofs", "cachefs",
 	     "swap", "tmpfs", "autofs");
 push(@fs, $smbmount) if ($smbmount);
+push(@fs, "udfs", "xmemfs") if ($gconfig{'os_version'} >= 8);
 return @fs;
 }
 
@@ -341,9 +343,12 @@ local(%fsmap);
 	  "cachefs","Caching Filesystem",
 	  "swap","Virtual Memory",
 	  "tmpfs","Ram Disk",
+	  "xmemfs","Large Ram Disk",
 	  "autofs","Automounter Filesystem",
 	  "proc","Process Image Filesystem",
 	  "fd","File Descriptor Filesystem",
+	  "mntfs","Filesystems List",
+	  "udfs","DVD Filesystem",
 	  "rumba","Windows Networking Filesystem");
 return $config{long_fstypes} && $fsmap{$_[0]} ? $fsmap{$_[0]} : uc($_[0]);
 }
@@ -367,7 +372,7 @@ else { return (2, 1, 0, 0); }
 sub multiple_mount
 {
 return ($_[0] eq "nfs" || $_[0] eq "tmpfs" || $_[0] eq "cachefs" ||
-        $_[0] eq "autofs" || $_[0] eq "lofs" || $_[0] eq "rumba");
+        $_[0] eq "autofs" || $_[0] eq "lofs" || $_[0] eq "rumba" || $_[0] eq "xmemfs");
 }
 
 
@@ -424,8 +429,8 @@ if ($_[0] eq "nfs") {
 		print "</td> </tr>\n";
 		}
 	}
-elsif ($_[0] eq "tmpfs") {
-	# Location is irrelevant for tmpfs filesystems
+elsif ($_[0] eq "tmpfs" || $_[0] eq "xmemfs") {
+	# Location is irrelevant for tmpfs and xmemfs filesystems
 	}
 elsif ($_[0] eq "ufs") {
 	# Mounted from a normal disk, raid (MD) device or from
@@ -434,7 +439,8 @@ elsif ($_[0] eq "ufs") {
 	print "<td colspan=3>\n";
 	&foreign_require("format", "format-lib.pl");
 
-	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/) {
+	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/ ||
+	    $_[1] =~ /^\/dev\/dsk\/c([0-9]+)d([0-9]+)s([0-9]+)$/) {
 		$ufs_dev = 0;
 		}
 	elsif ($_[1] eq "") {
@@ -473,7 +479,8 @@ elsif ($_[0] eq "swap") {
 	print "<td colspan=3>\n";
 	&foreign_require("format", "format-lib.pl");
 
-	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/) {
+	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/ ||
+	    $_[1] =~ /^\/dev\/dsk\/c([0-9]+)d([0-9]+)s([0-9]+)$/) {
 		$swap_dev = 0;
 		}
 	elsif ($_[1] eq "") {
@@ -496,9 +503,14 @@ elsif ($_[0] eq "swap") {
 		$swap_dev == 1 ? $_[1] : "";
 	print "</td> </tr>\n";
 	}
-elsif ($_[0] eq "hsfs") {
-	# Mounting a SCSI cdrom
-	print "<tr> <td valign=top><b>$text{'solaris_cdrom'}</b></td>\n";
+elsif ($_[0] eq "hsfs" || $_[0] eq "udfs") {
+	# Mounting a SCSI cdrom or DVD
+	if ($_[0] eq "hsfs") {
+		print "<tr> <td valign=top><b>$text{'solaris_cdrom'}</b></td>\n";
+		}
+	else {
+		print "<tr> <td valign=top><b>$text{'solaris_dvd'}</b></td>\n";
+		}
 	print "<td colspan=3>\n";
 	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/) {
 		$hsfs_dev = 0;
@@ -529,7 +541,8 @@ elsif ($_[0] eq "pcfs") {
 	print "<td colspan=3>\n";
 	&foreign_require("format", "format-lib.pl");
 
-	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/) {
+	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/ ||
+	    $_[1] =~ /^\/dev\/dsk\/c([0-9]+)d([0-9]+)s([0-9]+)$/) {
 		$pcfs_dev = 0;
 		}
 	elsif ($_[1] eq "") {
@@ -754,10 +767,21 @@ if ($_[0] eq "ufs") {
 
 	if ($gconfig{'os_version'} >= 7) {
 		print "<tr> <td><b>$text{'solaris_noatime'}</b></td> <td>\n";
-		printf "<input type=radio name=ufs_noatime value=0 %s> $text{'yes'}\n",
-			defined($options{'noatime'}) ? "" : "checked";
-		printf "<input type=radio name=ufs_noatime value=1 %s> $text{'no'}</td>\n",
-			defined($options{'noatime'}) ? "checked" : "";
+		if ($gconfig{'os_version'} >= 8) {
+			printf "<input type=radio name=ufs_noatime value=0 %s> $text{'solaris_immed'}\n",
+				defined($options{'nodfratime'}) ? "checked" : "";
+			printf "<input type=radio name=ufs_noatime value=1 %s> $text{'solaris_defer'}\n",
+				!defined($options{'nodfratime'}) &&
+				!defined($options{'noatime'}) ? "checked" : "";
+			printf "<input type=radio name=ufs_noatime value=2 %s> $text{'no'}</td>\n",
+				defined($options{'noatime'}) ? "checked" : "";
+			}
+		else {
+			printf "<input type=radio name=ufs_noatime value=0 %s> $text{'yes'}\n",
+				defined($options{'noatime'}) ? "" : "checked";
+			printf "<input type=radio name=ufs_noatime value=1 %s> $text{'no'}</td>\n",
+				defined($options{'noatime'}) ? "checked" : "";
+			}
 
 		print "<td><b>$text{'solaris_force'}</b></td> <td>\n";
 		printf "<input type=radio name=ufs_force value=1 %s> $text{'yes'}\n",
@@ -996,6 +1020,44 @@ if ($_[0] eq "rumba") {
 	printf "<input type=radio name=rumba_attr value=0 %s> $text{'no'}</td> </tr>\n",
 		defined($options{"attr"}) ? "" : "checked";
 	}
+if ($_[0] eq "udfs") {
+	# Solaris udfs is used for DVDs
+	print "<tr> <td><b>$text{'solaris_ro'}</b></td>\n";
+	printf "<td nowrap><input type=radio name=udfs_ro value=1 %s> $text{'yes'}\n",
+		defined($options{"ro"}) ? "checked" : "";
+	printf "<input type=radio name=udfs_ro value=0 %s> $text{'no'}</td>\n",
+		defined($options{"ro"}) ? "" : "checked";
+
+	print "<td><b>$text{'solaris_nosuid'}</b></td>\n";
+	printf "<td nowrap><input type=radio name=udfs_nosuid value=1 %s> $text{'yes'}\n",
+		defined($options{"nosuid"}) ? "checked" : "";
+	printf "<input type=radio name=udfs_nosuid value=0 %s> $text{'no'}</td> </tr>\n",
+		defined($options{"nosuid"}) ? "" : "checked";
+	}
+if ($_[0] eq "xmemfs") {
+	# Solaris xmemfs (virtual memory) filesystem.
+	print "<tr> <td><b>$text{'solaris_size'}</b>&nbsp;&nbsp;&nbsp;</td>\n";
+	printf"<td><input type=radio name=xmemfs_size_def value=1 %s> $text{'solaris_max'}\n",
+		defined($options{"size"}) ? "" : "checked";
+	printf"&nbsp;&nbsp;<input type=radio name=xmemfs_size_def value=0 %s>\n",
+		defined($options{"size"}) ? "checked" : "";
+	($tmpsz = $options{size}) =~ s/[A-z]+$//g;
+	print "<input name=xmemfs_size size=6 value=\"$tmpsz\">\n";
+	print "<select name=xmemfs_unit>\n";
+	printf "<option value=m %s> Mb\n",
+		$options{"size"} =~ /m$/ ? "selected" : "";
+	printf "<option value=k %s> Kb\n",
+		$options{"size"} =~ /k$/ ? "selected" : "";
+	printf "<option value=b %s> bytes\n",
+		$options{"size"} !~ /(k|m)$/ ? "selected" : "";
+	print "</select></td>\n";
+
+	print "<td><b>$text{'solaris_largebsize'}</b></td>\n";
+	printf "<td nowrap><input type=radio name=xmemfs_largebsize value=1 %s> $text{'yes'}\n",
+		defined($options{"largebsize"}) ? "checked" : "";
+	printf "<input type=radio name=xmemfs_largebsize value=0 %s> $text{'no'}</td> </tr>\n",
+		defined($options{"largebsize"}) ? "" : "checked";
+	}
 }
 
 
@@ -1082,7 +1144,7 @@ elsif ($_[0] eq "ufs" || $_[0] eq "pcfs") {
 	&fstyp_check($dv, $_[0]);
 	return $dv;
 	}
-elsif ($_[0] eq "hsfs") {
+elsif ($_[0] eq "hsfs" || $_[0] eq "udfs") {
 	# Get the device name
 	if ($in{ufs_dev} == 0) {
 		$in{ufs_c} =~ /^[0-9]+$/ ||
@@ -1142,6 +1204,10 @@ elsif ($_[0] eq "tmpfs") {
 	# Ram-disk filesystems have no location
 	return "swap";
 	}
+elsif ($_[0] eq "xmemfs") {
+	# Large ram-disk filesystems have no location
+	return "xmem";
+	}
 elsif ($_[0] eq "cachefs") {
 	# In order to check the location for the caching filesystem, we need
 	# to check the back filesystem
@@ -1357,8 +1423,16 @@ elsif ($_[0] eq "ufs") {
 				      $in{ufs_toosoon_units};
 		}
 	if ($gconfig{'os_version'} >= 7) {
-		delete($options{'noatime'});
-		$options{'noatime'} = "" if ($in{'ufs_noatime'});
+		if ($gconfig{'os_version'} >= 8) {
+			delete($options{'noatime'});
+			delete($options{'dfratime'}); delete($options{'nodfratime'});
+			if ($in{'ufs_noatime'} == 0) { $options{'nodfratime'} = ""; }
+			elsif ($in{'ufs_noatime'} == 2) { $options{'noatime'} = ""; }
+			}
+		else {
+			delete($options{'noatime'});
+			$options{'noatime'} = "" if ($in{'ufs_noatime'});
+			}
 
 		delete($options{'forcedirectio'});
 		delete($options{'noforcedirectio'});
@@ -1380,7 +1454,7 @@ elsif ($_[0] eq "swap") {
 elsif ($_[0] eq "pcfs") {
 	# PCFS has only 2 options
 	delete($options{'ro'}); delete($options{'rw'});
-	$options{'ro'} = "" if ($in{'pcfs_rp'});
+	$options{'ro'} = "" if ($in{'pcfs_ro'});
 
 	delete($options{'foldcase'}); delete($options{'nofoldcase'});
 	$options{'foldcase'} = "" if ($in{'pcfs_foldcase'});
@@ -1409,6 +1483,18 @@ elsif ($_[0] eq "tmpfs") {
 	delete($options{"nosuid"});
 	if ($in{'tmpfs_nosuid'}) { $options{"nosuid"} = ""; }
 	}
+elsif ($_[0] eq "xmemfs") {
+	# Large ram-disk filesystems have only two options
+	delete($options{"size"});
+	if (!$in{"xmemfs_size_def"}) {
+		$options{"size"} = "$in{xmemfs_size}$in{xmemfs_unit}";
+		}
+
+	delete($options{"largebsize"});
+	if ($in{'xmemfs_largebsize'}) { $options{"largebsize"} = ""; }
+
+	delete($options{'rw'});
+	}
 elsif ($_[0] eq "cachefs") {
 	# The caching filesystem has lots of options
 	$options{"backfstype"} = $in{"cfs_backfstype"};
@@ -1493,6 +1579,14 @@ elsif ($_[0] eq "rumba") {
 	delete($options{'noupper'});
 	if ($in{'rumba_noupper'}) { $options{'noupper'} = ""; }
 	}
+elsif ($_[0] eq "udfs") {
+	# The DVD filesystem has only 2 options
+	delete($options{'ro'}); delete($options{'rw'});
+	$options{'ro'} = "" if ($in{'udfs_ro'});
+
+	delete($options{"nosuid"});
+	if ($in{'udfs_nosuid'}) { $options{"nosuid"} = ""; }
+	}
 
 # Return options string
 foreach $k (keys %options) {
@@ -1559,4 +1653,9 @@ return $_[0] =~ /\/dev\/dsk\/c(\d+)t(\d+)d(\d+)s(\d+)$/ ?
 	$_[0];
 }
 
+sub files_to_lock
+{
+return ( $config{'fstab_file'}, $config{'autofs_file'} );
+}
+
 1;
