@@ -214,7 +214,7 @@ sub mount_dir
 local($out, $opts);
 if ($_[0] eq "swap") {
 	# Adding a swap device
-	$out = `swap -a $_[1] 2>&1`;
+	$out = &backquote_logged("swap -a $_[1] 2>&1");
 	if ($?) { return $out; }
 	}
 else {
@@ -230,7 +230,7 @@ else {
 				return "The directory $options{cachedir} ".
 				       "already exists. Delete it";
 				}
-			$out = `cfsadmin -c $options{cachedir} 2>&1`;
+			$out = &backquote_logged("cfsadmin -c $options{cachedir} 2>&1");
 			if ($?) { return $out; }
 			}
 		}
@@ -258,13 +258,13 @@ else {
 		 (defined($options{'readwrite'}) ? "-S " : "").
 		 (defined($options{'readonly'}) ? "-w " : "").
 		 (defined($options{'attr'}) ? "-e " : "");
-		$rv = system("rumba \"$shar\" $_[0] $opts >/tmp/$$.rumba 2>&1 </dev/null");
+		$rv = &system_logged("rumba \"$shar\" $_[0] $opts >/tmp/$$.rumba 2>&1 </dev/null");
 		$out = `cat /tmp/$$.rumba`; unlink("/tmp/$$.rumba");
 		if ($rv) { return "<pre>$out</pre> : rumba \"$shar\" $_[0] $opts"; }
 		}
 	else {
 		$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
-		$out = `mount -F $_[2] $opts -- $_[1] $_[0] 2>&1`;
+		$out = &backquote_logged("mount -F $_[2] $opts -- $_[1] $_[0] 2>&1");
 		if ($?) { return $out; }
 		}
 	}
@@ -278,20 +278,20 @@ return 0;
 sub unmount_dir
 {
 if ($_[0] eq "swap") {
-	$out = `swap -d $_[1] 2>&1`;
+	$out = &backquote_logged("swap -d $_[1] 2>&1");
 	}
 elsif ($_[2] eq "rumba") {
 	# kill the process (if nobody is in the directory)
 	$dir = $_[0];
 	if (`fuser -c $_[0] 2>/dev/null` =~ /\d/) { return "$_[0] is busy"; }
 	if (`cat /etc/mnttab` =~ /rumba-(\d+)\s+$dir\s+nfs/) {
-		kill('TERM', $1) || return "Failed to kill rumba";
+		&kill_logged('TERM', $1) || return "Failed to kill rumba";
 		}
 	else { return "Failed to find rumba pid"; }
 	sleep(1);
 	}
 else {
-	$out = `umount $_[0] 2>&1`;
+	$out = &backquote_logged("umount $_[0] 2>&1");
 	}
 if ($?) { return $out; }
 return 0;
@@ -1681,4 +1681,9 @@ sub device_name
 return $_[0];
 }
 
+sub files_to_lock
+{
+return ( $config{'fstab_file'}, $config{'autofs_file'} );
+}
+
 1;
