@@ -4,9 +4,9 @@
 do '../web-lib.pl';
 &init_config();
 $ENV{$gconfig{'ld_env'}} .= ':'.$config{'mysql_libs'};
+%access = &get_module_acl();
 $authstr = &make_authstr();
 $master_db = 'mysql';
-%access = &get_module_acl();
 
 @type_list = ('tinyint', 'smallint', 'mediumint', 'int', 'bigint',
 	    'float', 'double', 'decimal', 'date', 'datetime', 'timestamp',
@@ -16,8 +16,14 @@ $master_db = 'mysql';
 
 sub make_authstr
 {
-return ($config{'login'} ? " -u $config{'login'}" : "").
-       ($config{'pass'} ? " -p$config{'pass'}" : "");
+if ($access{'user'}) {
+	return " -u '$access{'user'}'".
+	       ($access{'pass'} ? " '-p$access{'pass'}'" : "");
+	}
+else {
+	return ($config{'login'} ? " -u '$config{'login'}'" : "").
+	       ($config{'pass'} ? " '-p$config{'pass'}'" : "");
+	}
 }
 
 # is_mysql_running()
@@ -85,40 +91,57 @@ open(DBS, "$config{'mysql'} $authstr -t $_[0] <$temp 2>&1 |");
 local $t = &parse_mysql_table(DBS);
 close(DBS);
 unlink($temp);
-if ($t =~ /^ERROR.*:(.*)/) {
+if ($t =~ /^ERROR[^:]*:(.*)/) {
 	&error(&text('esql', "<tt>$_[1]</tt>", "<tt>$1</tt>"));
 	}
 return $t;
 }
 
+# execute_sql_logged(database, command)
+sub execute_sql_logged
+{
+&additional_log('sql', $_[0], $_[1]);
+return &execute_sql(@_);
+}
+
 # parse_mysql_table(handle)
 # Given a filehandle, parses a text table in the format mysql uses
 sub parse_mysql_table
 {
 local $fh = $_[0];
-local $line;
+local ($line, $i, @edge);
 do {
-	# skip junk and first row of -'s
+	# skip to table top
 	$line = <$fh>;
 	return $line if ($line =~ /^ERROR/);
-	} while($line && $line !~ /^\|/);
-local @titles = &parse_mysql_line($line);
+	} while($line && $line !~ /^\+/);
+for($i=0; $i<length($line); $i++) {
+	push(@edge, $i) if (substr($line, $i, 1) eq '+');
+	}
+$line = <$fh>;		# skip first row of -'s
+local @titles = &parse_mysql_line($line, \@edge);
 $line = <$fh>;		# skip next row of -'s
 local @data;
 while(1) {
 	$line = <$fh>;
 	last if (!$line || $line !~ /^\|/);
-	push(@data, [ &parse_mysql_line($line) ]);
+	push(@data, [ &parse_mysql_line($line, \@edge) ]);
 	}
 return { 'titles' => \@titles,
 	 'data' => \@data };
 }
 
+# parse_mysql_line(line, &edges)
 sub parse_mysql_line
 {
-local @rv = split(/\|/, $_[0]);
-shift(@rv); pop(@rv);
-map { s/^\s+//; s/\s+$//; } @rv;
+local @rv;
+for($i=0; $i<@{$_[1]}-1; $i++) {
+	local $w = substr($_[0], $_[1]->[$i]+1,
+			  $_[1]->[$i+1] - $_[1]->[$i] - 2);
+	$w =~ s/^\s+//;
+	$w =~ s/\s+$//;
+	push(@rv, $w);
+	}
 return @rv;
 }
 
