@@ -49,10 +49,11 @@ if ($a->{'broadcast'}) { $cmd .= " broadcast $a->{'broadcast'}"; }
 if ($a->{'mtu'}) { $cmd .= " mtu $a->{'mtu'}"; }
 if ($a->{'up'}) { $cmd .= " up"; }
 else { $cmd .= " down"; }
-local $out = `$cmd 2>&1`;
+local $out = &backquote_logged("$cmd 2>&1");
 if ($?) { &error($out); }
 if ($a->{'ether'}) {
-	$out = `ifconfig $a->{'name'} hw ether $a->{'ether'} 2>&1`;
+	$out = &backquote_logged(
+		"ifconfig $a->{'name'} hw ether $a->{'ether'} 2>&1");
 	if ($?) { &error($out); }
 	}
 }
@@ -65,13 +66,13 @@ local $name = $_[0]->{'name'}.
 	      ($_[0]->{'virtual'} ne "" ? ":$_[0]->{'virtual'}" : "");
 if ($_[0]->{'virtual'} ne "") {
 	# Shutdown virtual interface by setting address to 0
-	local $out = `ifconfig $name 0 2>&1`;
+	local $out = &backquote_logged("ifconfig $name 0 2>&1");
 	if ($?) { &error($out); }
 	}
 local ($still) = grep { $_->{'fullname'} eq $name } &active_interfaces();
 if ($still) {
 	# Old version of ifconfig or non-virtual interface.. down it
-	local $out = `ifconfig $name down 2>&1`;
+	local $out = &backquote_logged("ifconfig $name down 2>&1");
 	if ($?) { &error($out); }
 	}
 }
@@ -83,7 +84,7 @@ sub boot_interfaces
 local(@rv, $f);
 opendir(CONF, $net_scripts_dir);
 while($f = readdir(CONF)) {
-	next if ($f !~ /^ifcfg-\S+/);
+	next if ($f !~ /^ifcfg-[a-z0-9:]+/);
 	local (%conf, $b);
 	&read_env_file("$net_scripts_dir/$f", \%conf);
 	$b->{'fullname'} = $conf{'DEVICE'};
@@ -113,6 +114,7 @@ sub save_interface
 local(%conf);
 local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
+&lock_file("$net_scripts_dir/ifcfg-$name");
 &read_env_file("$net_scripts_dir/ifcfg-$name", \%conf);
 $conf{'DEVICE'} = $name;
 $conf{'IPADDR'} = $_[0]->{'address'};
@@ -134,6 +136,7 @@ $conf{'ONBOOT'} = $_[0]->{'up'} ? "yes" : "no";
 $conf{'BOOTPROTO'} = $_[0]->{'bootp'} ? "bootp" :
 		     $_[0]->{'dhcp'} ? "dhcp" : "none";
 &write_env_file("$net_scripts_dir/ifcfg-$name", \%conf);
+&unlock_file("$net_scripts_dir/ifcfg-$name");
 }
 
 # delete_interface(&details)
@@ -142,7 +145,9 @@ sub delete_interface
 {
 local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
+&lock_file("$net_scripts_dir/ifcfg-$name");
 unlink("$net_scripts_dir/ifcfg-$name");
+&unlock_file("$net_scripts_dir/ifcfg-$name");
 }
 
 # iface_type(name)
@@ -189,6 +194,7 @@ local $dns;
 open(RESOLV, "/etc/resolv.conf");
 while(<RESOLV>) {
 	s/\r|\n//g;
+	s/#.*$//;
 	if (/nameserver\s+(.*)/) {
 		push(@{$dns->{'nameserver'}}, split(/\s+/, $1));
 		}
@@ -215,6 +221,7 @@ return $dns;
 # Writes out the resolv.conf and nsswitch.conf files
 sub save_dns_config
 {
+&lock_file("/etc/resolv.conf");
 open(RESOLV, "/etc/resolv.conf");
 local @resolv = <RESOLV>;
 close(RESOLV);
@@ -234,7 +241,9 @@ foreach (@resolv) {
 	print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
 	}
 close(RESOLV);
+&unlock_file("/etc/resolv.conf");
 
+&lock_file("/etc/nsswitch.conf");
 open(SWITCH, "/etc/nsswitch.conf");
 local @switch = <SWITCH>;
 close(SWITCH);
@@ -246,6 +255,7 @@ foreach (@switch) {
 	else { print SWITCH $_; }
 	}
 close(SWITCH);
+&unlock_file("/etc/nsswitch.conf");
 }
 
 $max_dns_servers = 3;
@@ -307,13 +317,17 @@ return &get_system_hostname();
 sub save_hostname
 {
 local %conf;
-system("hostname $_[0] >/dev/null 2>&1");
+&system_logged("hostname $_[0] >/dev/null 2>&1");
+&lock_file("/etc/HOSTNAME");
 open(HOST, ">/etc/HOSTNAME");
 print HOST $_[0],"\n";
 close(HOST);
+&unlock_file("/etc/HOSTNAME");
+&lock_file($network_config);
 &read_env_file($network_config, \%conf);
 $conf{'HOSTNAME'} = $_[0];
 &write_env_file($network_config, \%conf);
+&unlock_file($network_config);
 }
 
 # get_domainname()
@@ -415,6 +429,7 @@ print "</table></td> </tr>\n";
 sub parse_routing
 {
 local (%conf, @st);
+&lock_file($network_config);
 &read_env_file($network_config, \%conf);
 if ($in{'gateway_def'}) { $conf{'GATEWAY'} = ''; }
 elsif (!gethostbyname($in{'gateway'})) {
@@ -431,6 +446,7 @@ else { $conf{'GATEWAYDEV'} = $in{'gatewaydev'}; }
 if ($in{'forward'}) { $conf{'FORWARD_IPV4'} = 'yes'; }
 else { $conf{'FORWARD_IPV4'} = 'no'; }
 
+&lock_file($static_route_config);
 for($i=0; defined($dev = $in{"dev_$i"}); $i++) {
 	$net = $in{"net_$i"}; $netmask = $in{"netmask_$i"}; $gw = $in{"gw_$i"};
 	next if (!$dev && !$net && !$netmask && !$gw);
@@ -450,7 +466,9 @@ for($i=0; defined($dev = $in{"ldev_$i"}); $i++) {
 open(STATIC, ">$static_route_config");
 print STATIC @st;
 close(STATIC);
+&unlock_file($static_route_config);
 &write_env_file($network_config, \%conf);
+&unlock_file($network_config);
 }
 
 1;
