@@ -50,10 +50,11 @@ if ($a->{'broadcast'}) { $cmd .= " broadcast $a->{'broadcast'}"; }
 if ($a->{'mtu'}) { $cmd .= " mtu $a->{'mtu'}"; }
 if ($a->{'up'}) { $cmd .= " up"; }
 else { $cmd .= " down"; }
-local $out = `$cmd 2>&1`;
+local $out = &backquote_logged("$cmd 2>&1");
 if ($?) { &error($out); }
 if ($a->{'ether'}) {
-	$out = `ifconfig $a->{'name'} hw ether $a->{'ether'} 2>&1`;
+	$out = &backquote_logged(
+		"ifconfig $a->{'name'} hw ether $a->{'ether'} 2>&1");
 	if ($?) { &error($out); }
 	}
 }
@@ -65,7 +66,7 @@ sub deactivate_interface
 local $cmd = "ifconfig $a->{'name'}";
 if ($a->{'virtual'} ne "") { $cmd .= ":$a->{'virtual'}"; }
 $cmd .= " down";
-local $out = `$cmd 2>&1`;
+local $out = &backquote_logged("$cmd 2>&1");
 if ($?) { &error($out); }
 }
 
@@ -128,6 +129,7 @@ return @rv;
 sub save_interface
 {
 # Find existing interface, if any
+&lock_file($rc_config);
 local $rc = &parse_rc_config();
 local @boot = &boot_interfaces();
 local ($o, $old, $found);
@@ -171,12 +173,14 @@ else {
 	&save_rc_config($rc, "NETDEV$nnc", $fullname);
 	&save_rc_config($rc, "IFCONFIG$nnc", $ifconfig);
 	}
+&unlock_file($rc_config);
 }
 
 # delete_interface(&details)
 # Delete a boot-time interface
 sub delete_interface
 {
+&lock_file($rc_config);
 local $rc = &parse_rc_config();
 local @boot = &boot_interfaces();
 local $old;
@@ -193,8 +197,7 @@ foreach $old (@boot) {
 		&save_rc_config($rc, "IFCONFIG$nnc", "");
 		}
 	}
-
-
+&unlock_file($rc_config);
 }
 
 # iface_type(name)
@@ -262,10 +265,12 @@ return $dns;
 # Writes out the resolv.conf and nsswitch.conf files
 sub save_dns_config
 {
+&lock_file($rc_config);
 local $rc = &parse_rc_config();
 &save_rc_config($rc, "NAMESERVER", join(" ", @{$_[0]->{'nameserver'}}));
 &save_rc_config($rc, "SEARCHLIST", join(" ", @{$_[0]->{'domain'}}));
 
+&lock_file("/etc/nsswitch.conf");
 open(SWITCH, "/etc/nsswitch.conf");
 local @switch = <SWITCH>;
 close(SWITCH);
@@ -277,12 +282,14 @@ foreach (@switch) {
 	else { print SWITCH $_; }
 	}
 close(SWITCH);
+&unlock_file("/etc/nsswitch.conf");
 if ($_[0]->{'order'} =~ /nis/) {
 	&save_rc_config($rc, "USE_NIS_FOR_RESOLVING", "yes");
 	}
 else {
 	&save_rc_config($rc, "USE_NIS_FOR_RESOLVING", "no");
 	}
+&unlock_file($rc_config);
 }
 
 $max_dns_servers = 3;
@@ -344,8 +351,11 @@ return $rc->{'FQHOSTNAME'}->{'value'};
 # save_hostname(name)
 sub save_hostname
 {
+&system_logged("hostname $_[0] >/dev/null 2>&1");
+&lock_file($rc_config);
 local $rc = &parse_rc_config();
 &save_rc_config($rc, "FQHOSTNAME", $_[0]);
+&unlock_file($rc_config);
 }
 
 # get_domainname()
@@ -434,8 +444,11 @@ print "</table></td> </tr>\n";
 
 sub parse_routing
 {
+&lock_file($rc_config);
 local $rc = &parse_rc_config();
 &save_rc_config($rc, IP_FORWARD, $in{'forward'});
+&unlock_file($rc_config);
+&lock_file($route_conf);
 local $route = "# Generated by Webmin\n";
 for($i=0; defined($dev = $in{"lr_dev_$i"}); $i++) {
 	$net = $in{"lr_net_$i"}; $mask = $in{"lr_mask_$i"};
@@ -465,6 +478,7 @@ if (!$in{'default_def'}) {
 open(ROUTE, ">$route_conf");
 print ROUTE $route;
 close(ROUTE);
+&unlock_file($route_conf);
 }
 
 # parse_rc_config()
