@@ -54,10 +54,11 @@ else { $cmd .= " broadcast +"; }
 if ($a->{'mtu'}) { $cmd .= " mtu $a->{'mtu'}"; }
 if ($a->{'up'}) { $cmd .= " up"; }
 else { $cmd .= " down"; }
-local $out = `$cmd 2>&1`;
+local $out = &backquote_logged("$cmd 2>&1");
 if ($?) { &error($out); }
 if ($a->{'ether'}) {
-	$out = `ifconfig $a->{'name'} ether $a->{'ether'} 2>&1`;
+	$out = &backquote_logged(
+		"ifconfig $a->{'name'} ether $a->{'ether'} 2>&1");
 	if ($? && $out !~ /Device busy/) { &error($out); }
 	}
 }
@@ -73,7 +74,7 @@ if ($a->{'virtual'} eq "") {
 else {
 	$cmd = "ifconfig $a->{'name'}:$a->{'virtual'} 0.0.0.0 down";
 	}
-local $out = `$cmd 2>&1`;
+local $out = &backquote_logged("$cmd 2>&1");
 if ($?) { &error($out); }
 }
 
@@ -147,11 +148,13 @@ sub save_interface
 {
 local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
+&lock_file("/etc/hostname.$name");
 open(IFACE, ">/etc/hostname.$name");
 if (!$_[0]->{'dhcp'}) {
 	print IFACE $_[0]->{'address'},"\n";
 	}
 close(IFACE);
+&unlock_file("/etc/hostname.$name");
 }
 
 # delete_interface(&details)
@@ -160,7 +163,9 @@ sub delete_interface
 {
 local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
+&lock_file("/etc/hostname.$name");
 unlink("/etc/hostname.$name");
+&unlock_file("/etc/hostname.$name");
 }
 
 # iface_type(name)
@@ -231,6 +236,7 @@ return $dns;
 # Writes out the resolv.conf and nsswitch.conf files
 sub save_dns_config
 {
+&lock_file("/etc/resolv.conf");
 open(RESOLV, "/etc/resolv.conf");
 local @resolv = <RESOLV>;
 close(RESOLV);
@@ -250,7 +256,9 @@ foreach (@resolv) {
 	print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
 	}
 close(RESOLV);
+&unlock_file("/etc/resolv.conf");
 
+&lock_file("/etc/nsswitch.conf");
 open(SWITCH, "/etc/nsswitch.conf");
 local @switch = <SWITCH>;
 close(SWITCH);
@@ -262,6 +270,7 @@ foreach (@switch) {
 	else { print SWITCH $_; }
 	}
 close(SWITCH);
+&unlock_file("/etc/nsswitch.conf");
 }
 
 $max_dns_servers = 3;
@@ -320,7 +329,7 @@ return &get_system_hostname();
 # save_hostname(name)
 sub save_hostname
 {
-system("hostname $_[0] >/dev/null 2>&1");
+&system_logged("hostname $_[0] >/dev/null 2>&1");
 }
 
 # get_domainname()
@@ -376,13 +385,17 @@ local @defrt = split(/\s+/, $in{'defrt'});
 foreach $d (@defrt) {
 	gethostbyname($d) || &error(&text('routes_edefault', $d));
 	}
+&lock_file("/etc/defaultrouter");
 if (@defrt) {
 	open(DEFRT, ">/etc/defaultrouter");
 	foreach $d (@defrt) { print DEFRT $d,"\n"; }
 	close(DEFRT);
 	}
 else { unlink("/etc/defaultrouter"); }
+&unlock_file("/etc/defaultrouter");
 
+&lock_file("/etc/gateways");
+&lock_file("/etc/notrouter");
 if ($in{'router'} == 0) {
 	&create_empty_file("/etc/gateways");
 	unlink("/etc/notrouter");
@@ -395,6 +408,8 @@ else {
 	unlink("/etc/gateways");
 	unlink("/etc/notrouter");
 	}
+&unlock_file("/etc/gateways");
+&unlock_file("/etc/notrouter");
 }
 
 # create_empty_file(filename)
