@@ -15,7 +15,7 @@ for($i=0; $line=<PS>; $i++) {
 	$plist[$i]->{"size"} = $8;
 	$plist[$i]->{"time"} = $13;
 	$plist[$i]->{"nice"} = $6;
-	$plist[$i]->{"_tty"} = $12 eq '?' ? 'None' : "/dev/tty$12";
+	$plist[$i]->{"_tty"} = $12 eq '?' ? $text{'edit_none'} : "/dev/tty$12";
 	$plist[$i]->{"args"} = $14;
 	$pidmap{$3} = $plist[$i];
 	}
