@@ -121,6 +121,14 @@ return $got;
 $SIG{'CHLD'} = $chld;
 }
 
+# safe_process_exec_logged(..)
+# Like safe_process_exec, but also logs the command
+sub safe_process_exec_logged
+{
+&additional_log('exec', undef, $_[0]);
+return &safe_process_exec(@_);
+}
+
 sub safe_exec_reaper
 {
 local $xp;
@@ -153,6 +161,14 @@ close($ttyfh);
 return ($ptyfh, $pid);
 }
 
+# pty_process_exec_logged(..)
+# Like pty_process_exec, but logs the command as well
+sub pty_process_exec_logged
+{
+&additional_log('exec', undef, $_[0]);
+return &pty_process_exec(@_);
+}
+
 # find_process(name)
 # Returns an array of all processes matching some name
 sub find_process
