@@ -81,7 +81,7 @@ $ENV{'QUOTA_SBLOCKS'} = $_[2];
 $ENV{'QUOTA_HBLOCKS'} = $_[3];
 $ENV{'QUOTA_SFILES'} = $_[4];
 $ENV{'QUOTA_HFILES'} = $_[5];
-system("$config{'user_edquota_command'} $_[0]");
+&system_logged("$config{'user_edquota_command'} $_[0]");
 }
 
 # edit_group_quota(group, filesys, sblocks, hblocks, sfiles, hfiles)
@@ -94,7 +94,7 @@ $ENV{'QUOTA_SBLOCKS'} = $_[2];
 $ENV{'QUOTA_HBLOCKS'} = $_[3];
 $ENV{'QUOTA_SFILES'} = $_[4];
 $ENV{'QUOTA_HFILES'} = $_[5];
-system("$config{'group_edquota_command'} $_[0]");
+&system_logged("$config{'group_edquota_command'} $_[0]");
 }
 
 # edit_user_grace(filesystem, btime, bunits, ftime, funits)
@@ -107,7 +107,7 @@ $ENV{'QUOTA_BTIME'} = $_[1];
 $ENV{'QUOTA_BUNITS'} = $_[2];
 $ENV{'QUOTA_FTIME'} = $_[3];
 $ENV{'QUOTA_FUNITS'} = $_[4];
-system($config{'user_grace_command'});
+&system_logged($config{'user_grace_command'});
 }
 
 # edit_group_grace(filesystem, btime, bunits, ftime, funits)
@@ -120,13 +120,13 @@ $ENV{'QUOTA_BTIME'} = $_[1];
 $ENV{'QUOTA_BUNITS'} = $_[2];
 $ENV{'QUOTA_FTIME'} = $_[3];
 $ENV{'QUOTA_FUNITS'} = $_[4];
-system($config{'group_grace_command'});
+&system_logged($config{'group_grace_command'});
 }
 
 # quota_input(name)
 sub quota_input
 {
-printf "<td><input type=radio name=$_[0]_def value=1 %s> $text{'quota-lib_0'}\n",
+printf "<td><input type=radio name=$_[0]_def value=1 %s> $text{'quota_unlimited'}\n",
 	$_[1]==0 ? "checked" : "";
 printf "<input type=radio name=$_[0]_def value=0 %s>\n",
 	$_[1]==0 ? "" : "checked";
