@@ -136,7 +136,7 @@ open(AFILE, ">>".$_[1]->[0]);
 print AFILE $_[0]->{'enabled'} ? "" : "# ", $_[0]->{'name'}, ": ",
 	    join(',', map { /\s/ ? "\"$_\"" : $_ } @{$_[0]->{'values'}}),"\n";
 close(AFILE);
-system("newaliases >/dev/null 2>&1") if (!$_[2]);
+&system_logged("newaliases >/dev/null 2>&1") if (!$_[2]);
 }
 
 # delete_alias(&details)
@@ -146,7 +146,7 @@ local $lref = &read_file_lines($_[0]->{'file'});
 local $len = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
 splice(@$lref, $_[0]->{'line'}, $len);
 &flush_file_lines();
-system("newaliases >/dev/null 2>&1");
+&system_logged("newaliases >/dev/null 2>&1");
 }
 
 # modify_alias(&old, &details)
@@ -159,7 +159,7 @@ local $lref = &read_file_lines($_[0]->{'file'});
 local $len = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
 splice(@$lref, $_[0]->{'line'}, $len, $str);
 &flush_file_lines();
-system("newaliases >/dev/null 2>&1");
+&system_logged("newaliases >/dev/null 2>&1");
 }
 
 # alias_type(string)
@@ -185,5 +185,21 @@ else {
 return wantarray ? @rv : $rv[0];
 }
 
+# lock_alias_files(&files)
+sub lock_alias_files
+{
+foreach $f (@{$_[0]}) {
+	&lock_file($f);
+	}
+}
+
+# unlock_alias_files(&files)
+sub unlock_alias_files
+{
+foreach $f (@{$_[0]}) {
+	&unlock_file($f);
+	}
+}
+
 1;
 
