@@ -3,7 +3,8 @@
 # Simple autoreply script
 
 # read sendmail module config
-$0 =~ /^(.*)\/[^\/]+$/;
+$p = -l $0 ? readlink($0) : $0;
+$p =~ /^(.*)\/[^\/]+$/;
 open(CONF, "$1/config");
 while(<CONF>) {
 	if (/^(\S+)=(.*)/) {
