@@ -23,22 +23,21 @@ open(MAIL, "$config{'mail_dir'}/$_[0]");
 for($i=$start; $i<=$end; $i++) {
 	local ($mail, $line, @headers);
 	seek(MAIL, $index[$i]->[0], 0);
-	$line = <MAIL>;	# skip From line
 
-	# read mail headers
+	# read RFC822 headers
 	$mail->{'line'} = $index[$i]->[1];
-	local $lnum = 1;
+	local $lnum = 0;
 	while(1) {
 		$lnum++;
 		($line = <MAIL>) =~ s/\r|\n//g;
 		$mail->{'size'} += length($line);
+		last if ($line =~ /^$/);
 		if ($line =~ /^(\S+):\s*(.*)/) {
 			push(@headers, [ $1, $2 ]);
 			}
 		elsif ($line =~ /^(\s+.*)/) {
-			$headers[$#headers]->[1] .= $1;
+			$headers[$#headers]->[1] .= $1 unless($#headers < 0);
 			}
-		else { last; }
 		}
 	$mail->{'headers'} = \@headers;
 	foreach $h (@headers) {
@@ -70,22 +69,21 @@ open(MAIL, "$config{'mail_dir'}/$_[0]");
 for($i=@index-1; $i>=0; $i--) {
 	local ($mail, $line, @headers);
 	seek(MAIL, $index[$i]->[0], 0);
-	$line = <MAIL>;	# skip From line
 
 	# read mail headers
 	$mail->{'line'} = $index[$i]->[1];
-	local $lnum = 1;
+	local $lnum = 0;
 	while(1) {
 		$lnum++;
 		($line = <MAIL>) =~ s/\r|\n//g;
 		$mail->{'size'} += length($line);
+		last if ($line =~ /^$/);
 		if ($line =~ /^(\S+):\s*(.*)/) {
 			push(@headers, [ $1, $2 ]);
 			}
 		elsif ($line =~ /^(\s+.*)/) {
 			$headers[$#headers]->[1] .= $1;
 			}
-		else { last; }
 		}
 	$mail->{'headers'} = \@headers;
 	foreach $h (@headers) {
@@ -528,16 +526,20 @@ if ($_[0] =~ /^(\S+),\s+0*(\d+)\s+(\S+)\s+(\d+)\s+(\d+):(\d+)/) {
 return $_[0];
 }
 
+# simplify_from(from)
+# Simplifies a From: address for display in the list mail
 sub simplify_from
 {
-local $rv = $_[0];
+local $rv = &decode_mimewords($_[0]);
 $rv =~ s/<[^>]+>//g;
-return &html_escape($rv);
+return $rv ? &html_escape($rv) : $text{'mail_none'};
 }
 
+# simplify_subject(subject)
 sub simplify_subject
 {
-return $_[0] =~ /\S/ ? &html_escape($_[0]) : "<br>";
+local $rv = &decode_mimewords($_[0]);
+return $rv =~ /\S/ ? &html_escape($rv) : "<br>";
 }
 
 # quoted_decode(text)
@@ -549,4 +551,79 @@ $t =~ s/=(\S\S)/pack("c",hex($1))/ge;
 return $t;
 }
 
+sub decode_mimewords {
+    my $encstr = shift;
+    my %params = @_;
+    my @tokens;
+    $@ = '';           ### error-return
+
+    ### Collapse boundaries between adjacent encoded words:
+    $encstr =~ s{(\?\=)\r?\n[ \t](\=\?)}{$1$2}gs;
+    pos($encstr) = 0;
+    ### print STDOUT "ENC = [", $encstr, "]\n";
+
+    ### Decode:
+    my ($charset, $encoding, $enc, $dec);
+    while (1) {
+	last if (pos($encstr) >= length($encstr));
+	my $pos = pos($encstr);               ### save it
+
+	### Case 1: are we looking at "=?..?..?="?
+	if ($encstr =~    m{\G             # from where we left off..
+			    =\?([^?]*)     # "=?" + charset +
+			     \?([bq])      #  "?" + encoding +
+			     \?([^?]+)     #  "?" + data maybe with spcs +
+			     \?=           #  "?="
+			    }xgi) {
+	    ($charset, $encoding, $enc) = ($1, lc($2), $3);
+	    $dec = (($encoding eq 'q') ? _decode_Q($enc) : _decode_B($enc));
+	    push @tokens, [$dec, $charset];
+	    next;
+	}
+
+	### Case 2: are we looking at a bad "=?..." prefix? 
+	### We need this to detect problems for case 3, which stops at "=?":
+	pos($encstr) = $pos;               # reset the pointer.
+	if ($encstr =~ m{\G=\?}xg) {
+	    $@ .= qq|unterminated "=?..?..?=" in "$encstr" (pos $pos)\n|;
+	    push @tokens, ['=?'];
+	    next;
+	}
+
+	### Case 3: are we looking at ordinary text?
+	pos($encstr) = $pos;               # reset the pointer.
+	if ($encstr =~ m{\G                # from where we left off...
+			 ([\x00-\xFF]*?    #   shortest possible string,
+			  \n*)             #   followed by 0 or more NLs,
+		         (?=(\Z|=\?))      # terminated by "=?" or EOS
+			}xg) {
+	    length($1) or die "MIME::Words: internal logic err: empty token\n";
+	    push @tokens, [$1];
+	    next;
+	}
+
+	### Case 4: bug!
+	die "MIME::Words: unexpected case:\n($encstr) pos $pos\n\t".
+	    "Please alert developer.\n";
+    }
+    return join('',map {$_->[0]} @tokens);
+}
+
+# _decode_Q STRING
+#     Private: used by _decode_header() to decode "Q" encoding, which is
+#     almost, but not exactly, quoted-printable.  :-P
+sub _decode_Q {
+    my $str = shift;
+    $str =~ s/_/\x20/g;                                # RFC-1522, Q rule 2
+    $str =~ s/=([\da-fA-F]{2})/pack("C", hex($1))/ge;  # RFC-1522, Q rule 1
+    $str;
+}
+
+# _decode_B STRING
+#     Private: used by _decode_header() to decode "B" encoding.
+sub _decode_B {
+    my $str = shift;
+    &decode_base64($str);
+}
+
 1;
