@@ -28,7 +28,7 @@ if ($dir) {
 		chown($_[0]->{'uid'}, $st[5], $mfile);
 		}
 	if ($_[0]->{'olduser'} ne $_[0]->{'user'}) {
-		rename($mfile, "$dir/$_[0]->{'user'}");
+		&rename_logged($mfile, "$dir/$_[0]->{'user'}");
 		}
 	}
 }
