@@ -29,13 +29,17 @@ return $serv;
 # save_server(&server)
 sub save_server
 {
+&lock_file("$module_config_directory/$_[0]->{'id'}.serv");
 &write_file("$module_config_directory/$_[0]->{'id'}.serv", $_[0]);
+&unlock_file("$module_config_directory/$_[0]->{'id'}.serv");
 }
 
 # delete_server(id)
 sub delete_server
 {
+&lock_file("$module_config_directory/$_[0].serv");
 unlink("$module_config_directory/$_[0].serv");
+&unlock_file("$module_config_directory/$_[0].serv");
 }
 
 @server_types = ( [ 'caldera', 'OpenLinux' ],
