@@ -50,9 +50,9 @@ fi
 if [ ! -d $config_dir ]; then
 	mkdir $config_dir;
 	if [ $? != 0 ]; then
-		echo "ERROR: Failed to create directory $config_dir";
-		echo "";
-		exit 2;
+		echo "ERROR: Failed to create directory $config_dir"
+		echo ""
+		exit 2
 	fi
 fi
 if [ -r "$config_dir/config" ]; then
@@ -109,6 +109,14 @@ if [ "$upgrading" = 1 ]; then
 		echo ""
 	fi
 else
+	# Config directory exists .. make sure it is not in use
+	ls $config_dir | grep -v rpmsave >/dev/null 2>&1
+	if [ "$?" = "0" ]; then
+		echo "ERROR: Config directory $config_dir is not empty"
+		echo ""
+		exit 2
+	fi
+
 	# Ask for log directory
 	printf "Log file directory [/var/webmin]: "
 	if [ "$var_dir" = "" ]; then
@@ -381,6 +389,7 @@ echo "Creating start and stop scripts.."
 rm -f $config_dir/stop $config_dir/start
 echo "#!/bin/sh" >>$config_dir/start
 echo "echo Starting Webmin server in $wadir" >>$config_dir/start
+echo "trap '' 1" >>$config_dir/start
 echo "exec $wadir/miniserv.pl $config_dir/miniserv.conf &" >>$config_dir/start
 
 echo "#!/bin/sh" >>$config_dir/stop
@@ -401,6 +410,27 @@ if [ "$upgrading" != 1 ]; then
 	echo "os_version=$os_version" >> $config_dir/config
 	echo "real_os_type=$real_os_type" >> $config_dir/config
 	echo "real_os_version=$real_os_version" >> $config_dir/config
+	if [ -r /etc/system.cnf ]; then
+		# Found a caldera system config file .. get the language
+		source /etc/system.cnf
+		if [ "$CONF_LST_LANG" = "us" ]; then
+			CONF_LST_LANG=en
+		elif [ "$CONF_LST_LANG" = "uk" ]; then
+			CONF_LST_LANG=en
+		fi
+		grep "lang=$CONF_LST_LANG," $wadir/lang_list.txt >/dev/null 2>&1
+		if [ "$?" = 0 ]; then
+			echo "lang=$CONF_LST_LANG" >> $config_dir/config
+		fi
+	fi
+else
+	# one-off hack to set log variable in config from miniserv.conf
+	grep log= $config_dir/config >/dev/null
+	if [ "$?" = "1" ]; then
+		grep log= $config_dir/miniserv.conf >> $config_dir/config
+		grep logtime= $config_dir/miniserv.conf >> $config_dir/config
+		grep logclear= $config_dir/miniserv.conf >> $config_dir/config
+	fi
 fi
 echo $ver > $config_dir/version
 echo "..done"
@@ -440,6 +470,8 @@ fi
 echo "Changing ownership and permissions .."
 chown -R root:bin $config_dir
 chmod -R og-rw $config_dir
+chmod 755 /etc/webmin/sendmail/config >/dev/null 2>&1
+chmod 755 /etc/webmin/sendmail/autoreply.pl >/dev/null 2>&1
 if [ "$nochown" = "" ]; then
 	chown -R root:bin $wadir
 	#chmod -R og-rwx $wadir
