@@ -158,7 +158,7 @@ sub install_package
 {
 local $args = ($in{"scripts"} ? " -I" : "").
 	      ($in{"force"} ? " -f" : "");
-local $out = `pkg_add $args $_[0] 2>&1`;
+local $out = &backquote_logged("pkg_add $args $_[0] 2>&1");
 if ($?) {
 	return "<pre>$out</pre>";
 	}
@@ -169,7 +169,7 @@ return undef;
 # Totally remove some package
 sub delete_package
 {
-local $out = `pkg_delete $_[0] 2>&1`;
+local $out = &backquote_logged("pkg_delete $_[0] 2>&1");
 if ($?) { return "<pre>$out</pre>"; }
 return undef;
 }
