@@ -235,7 +235,7 @@ if ($in{'root'} =~ /^\/.+/) {
 		}
 	$opts .= " -r $in{'root'}";
 	}
-$out = `swinstall -s $_[0] $opts $_[1] 2>&1`;
+$out = &backquote_logged("swinstall -s $_[0] $opts $_[1] 2>&1");
 if ($?) { return "<pre>$out</pre>"; }
 return undef;
 }
@@ -353,7 +353,7 @@ else { undef(%file); return 0; }
 # Attempt to remove some package
 sub delete_package
 {
-$out = `swremove $_[0] 2>&1`;
+$out = &backquote_logged("swremove $_[0] 2>&1");
 if ($out) { return "<pre>$out</pre>"; }
 return undef;
 }
