@@ -4,6 +4,7 @@
 do '../web-lib.pl';
 &init_config();
 do './parser-lib.pl';
+%access = &get_module_acl();
 
 # Get the squid version
 if (open(VERSION, "$module_config_directory/version")) {
@@ -395,5 +396,18 @@ else { push(@list, $config{'cache_dir'}); }
 system("chown -Rf $_[0]:$_[1] ".join(" ",@list)." >/dev/null 2>&1");
 }
 
+# can_access(file)
+sub can_access
+{
+local @f = grep { $_ ne '' } split(/\//, $_[0]);
+return 1 if ($access{'root'} eq '/' || $_[0] eq '/');
+local @a = grep { $_ ne '' } split(/\//, $access{'root'});
+local $i;
+for($i=0; $i<@a && $i<@f; $i++) {
+	return 0 if ($a[$i] ne $f[$i]);
+	}
+return 1;
+}
+
 1;
 
