@@ -30,11 +30,14 @@ return 1;
 # Write out the contents of an associative array as name=value lines
 sub write_file
 {
-local($arr);
-$arr = $_[1];
-open(ARFILE, "> $_[0]");
-foreach $k (keys %$arr) {
-        print ARFILE "$k=$$arr{$k}\n";
+local(%old, @order);
+&read_file($_[0], \%old, \@order);
+open(ARFILE, ">$_[0]");
+foreach $k (@order) {
+        print ARFILE $k,"=",$_[1]->{$k},"\n" if (exists($_[1]->{$k}));
+	}
+foreach $k (keys %{$_[1]}) {
+        print ARFILE $k,"=",$_[1]->{$k},"\n" if (!exists($old{$k}));
         }
 close(ARFILE);
 }
@@ -48,50 +51,10 @@ $tmp = $_[0];
 $tmp =~ s/&/&amp;/g;
 $tmp =~ s/</&lt;/g;
 $tmp =~ s/>/&gt;/g;
-$tmp =~ s/\"/&#34;/g;
+$tmp =~ s/\"/&quot;/g;
 return $tmp;
 }
 
-# obtain_lock
-# Get a lock on a file, or wait.
-# If the file is locked, and the locking process no longer exists ignore the
-# lock and carry on.
-sub obtain_lock {
-  while(stat("$_[0].lock")) {
-	sleep(1);
-	open(LOCK, "$_[0].lock");
-	chop($lpid = <LOCK>);
-	close(LOCK);
-	if ($lpid && !kill(0, "/proc/$lpid")) {
-		# The process holding this lock is gone!
-		# print STDERR "Process $lpid does not exist\n";
-		last;
-		}
-	}
-  open(LOCK,">$_[0].lock");
-  print LOCK "$$\n";
-  close(LOCK);
-}
-
-# release_lock
-# Release the lock we are holding on a file
-sub release_lock {
-  unlink("$_[0].lock");
-}
-
-# test_lock
-# Is this file locked?
-sub test_lock
-{
-if (-r "$_[0].lock") {
-	open(LOCK, "$_[0].lock");
-	<LOCK> =~ /(\d+)/;
-	close(LOCK);
-	if ($1 && kill(0, $1)) { return 1; }
-	}
-return 0;
-}
-
 # tempname
 # Returns a mostly random temporary file name
 sub tempname
@@ -106,8 +69,8 @@ while(1) {
 	mkdir("/tmp/.webmin", 0755) || next;
 	chown(0, 0, "/tmp/.webmin");
 	}
-$tempfilecount++;
-return "/tmp/.webmin/".$$."_".$tempfilecount."_".$scriptname;
+$main::tempfilecount++;
+return "/tmp/.webmin/".$$."_".$main::tempfilecount."_".$scriptname;
 }
 
 # trunc
@@ -274,8 +237,9 @@ local $i;
 if ($ENV{'REQUEST_METHOD'} eq 'POST') {
 	read(STDIN, $in, $ENV{'CONTENT_LENGTH'});
 	}
-else {
-	$in = $ENV{'QUERY_STRING'};
+if ($ENV{'QUERY_STRING'}) {
+	if ($in) { $in .= "&".$ENV{'QUERY_STRING'}; }
+	else { $in = $ENV{'QUERY_STRING'}; }
 	}
 @in = split(/\&/, $in);
 foreach $i (@in) {
@@ -365,7 +329,12 @@ if (@_ > 1) {
 		print "<td align=center width=70%>";
 		foreach $l (split(//, $title)) {
 			$ll = ord($l);
-			print "<img src=/images/letters/$ll.gif align=bottom>";
+			if ($ll > 127 && $lang->{'charset'}) {
+				print "<img src=/images/letters/$ll.$lang->{'charset'}.gif align=bottom>";
+				}
+			else {
+				print "<img src=/images/letters/$ll.gif align=bottom>";
+				}
 			}
 		print "</td>\n";
 		}
@@ -534,14 +503,14 @@ return sprintf "%d/%s/%d %2.2d:%2.2d",
 		$tm[3], $make_date_marr[$tm[4]], $tm[5]+1900, $tm[2], $tm[1];
 }
 
-# file_chooser_button(input, type, [form])
+# file_chooser_button(input, type, [form], [chroot])
 # Return HTML for a file chooser button, if the browser supports Javascript.
 # Type values are 0 for file or directory, or 1 for directory only
 sub file_chooser_button
 {
-local($form);
-$form = @_ > 2 ? $_[2] : 0;
-return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"/chooser.cgi?type=$_[1]&file=\"+ifield.value, \"chooser\", \"toolbar=no,menubar=no,scrollbar=no,width=400,height=300\"); chooser.ifield = ifield' value=\"...\">\n";
+local $form = @_ > 2 ? $_[2] : 0;
+local $chroot = @_ > 3 ? $_[3] : "/";
+return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"/chooser.cgi?type=$_[1]&chroot=$chroot&file=\"+ifield.value, \"chooser\", \"toolbar=no,menubar=no,scrollbar=no,width=400,height=300\"); chooser.ifield = ifield' value=\"...\">\n";
 }
 
 # read_acl(&array, &array)
@@ -600,7 +569,7 @@ open(PID, $miniserv{'pidfile'}) || &error("Failed to open pid file");
 chop($pid = <PID>);
 close(PID);
 if (!$pid) { &error("Invalid pid file"); }
-kill('HUP', $pid);
+&kill_logged('HUP', $pid);
 
 # wait for miniserv to come back up
 $addr = inet_aton($miniserv{'bind'} ? $miniserv{'bind'} : "127.0.0.1");
@@ -1004,6 +973,7 @@ local $m = $_[2] ? $_[2] : $module_name;
 #  $scriptname - Base name of the current perl script
 #  $module_name - The name of the current module
 #  $module_config_directory - The config directory for this module
+#  $webmin_logfile - The detailed logfile for webmin
 sub init_config
 {
 # Read the webmin global config file. This contains the OS type and version,
@@ -1044,6 +1014,8 @@ $cb = defined($gconfig{'cs_table'}) ? "bgcolor=#$gconfig{'cs_table'}"
 				    : "bgcolor=#cccccc";
 $0 =~ /([^\/]+)$/;
 $scriptname = $1;
+$webmin_logfile = $gconfig{'webmin_log'} ? $gconfig{'webmin_log'}
+					 : "$ENV{'WEBMIN_VAR'}/webmin.log";
 
 # Load language strings into %text
 local $u = $ENV{'REMOTE_USER'};
@@ -1066,8 +1038,8 @@ if ($module_name && !$main::no_acl_check && !$ENV{'FOREIGN_MODULE_NAME'}) {
 if ($0 && $0 !~ /index\.cgi$/ && $0 !~ /referer_save\.cgi$/ &&
     !$gconfig{'referer'} &&
     $ENV{'MINISERV_CONFIG'} &&
-    $ENV{'HTTP_REFERER'} =~ /^http:\/\/([^\/]+)\// &&
-    $1 ne $ENV{'HTTP_HOST'}) {
+    $ENV{'HTTP_REFERER'} =~ /^http:\/\/([^:]+:[^@]+@)?([^\/]+)\// &&
+    $2 ne $ENV{'HTTP_HOST'}) {
 	# Looks like a link from elsewhere ..
 	&header($text{'referer_title'}, "");
 	print "<hr><center>\n";
@@ -1131,6 +1103,12 @@ for($i=1; $i<@_; $i++) {
 return $rv;
 }
 
+# terror(text params)
+sub terror
+{
+&error(&text(@_));
+}
+
 # encode_base64(string)
 # Encodes a string into base64 format
 sub encode_base64
@@ -1141,12 +1119,33 @@ sub encode_base64
         $res .= substr(pack('u', $1), 1)."\n";
         chop($res);
     }
-    $res =~ tr|` -_|AA-Za-z0-9+/|;
+    $res =~ tr|\` -_|AA-Za-z0-9+/|;
     local $padding = (3 - length($_[0]) % 3) % 3;
     $res =~ s/.{$padding}$/'=' x $padding/e if ($padding);
     return $res;
 }
 
+# decode_base64(string)
+# Converts a base64 string into plain text
+sub decode_base64
+{
+    local $str = $_[0];
+    local $res;
+ 
+    $str =~ tr|A-Za-z0-9+=/||cd;            # remove non-base64 chars
+    if (length($str) % 4) {
+        print STDERR "Length of base64 data not a multiple of 4\n";
+	return undef;
+    }
+    $str =~ s/=+$//;                        # remove padding
+    $str =~ tr|A-Za-z0-9+/| -_|;            # convert to uuencoded format
+    while ($str =~ /(.{1,60})/gs) {
+        my $len = chr(32 + length($1)*3/4); # compute length byte
+        $res .= unpack("u", $len . $1 );    # uudecode
+    }
+    return $res;
+}
+
 # get_module_info(module, [noclone])
 # Returns a hash containg a module name, desc and os_support
 sub get_module_info
@@ -1227,16 +1226,337 @@ foreach $k (keys %{$_[1]}) {
 close(FILE);
 }
 
-# lock_file(filename)
-# Wait until a lock is obtained on the given file
+# lock_file(filename, [readonly], [forcefile])
+# Lock a file for exclusive access. If the file is already locked, spin
+# until it is freed. This version uses a .lock file, which is not very reliable.
 sub lock_file
 {
+return if (!$_[0] || defined($main::locked_file_list{$_[0]}));
+while(1) {
+	local $pid;
+	if (open(LOCKING, "$_[0].lock")) {
+		chop($pid = <LOCKING>);
+		close(LOCKING);
+		}
+	if (!$pid || !kill(0, $pid) || $pid == $$) {
+		# got the lock!
+		open(LOCKING, ">$_[0].lock");
+		print LOCKING $$,"\n";
+		close(LOCKING);
+		$main::locked_file_list{$_[0]} = int($_[1]);
+		if ($gconfig{'logfiles'} && !$_[1]) {
+			# Grab a copy of this file for later diffing
+			local $lnk;
+			$main::locked_file_data{$_[0]} = undef;
+			if (-d $_[0]) {
+				$main::locked_file_type{$_[0]} = 1;
+				$main::locked_file_data{$_[0]} = '';
+				}
+			elsif (!$_[2] && ($lnk = readlink($_[0]))) {
+				$main::locked_file_type{$_[0]} = 2;
+				$main::locked_file_data{$_[0]} = $lnk;
+				}
+			elsif (open(ORIGFILE, $_[0])) {
+				$main::locked_file_type{$_[0]} = 0;
+				$main::locked_file_data{$_[0]} = '';
+				while(<ORIGFILE>) {
+					$main::locked_file_data{$_[0]} .= $_;
+					}
+				close(ORIGFILE);
+				}
+			}
+		last;
+		}
+	sleep(1);
+	}
+}
+
+# old_lock_file(filename, [readonly])
+# Lock a file for exclusive access. If the file is already locked, spin
+# until it is freed
+# This version didn't work on NFS !!
+sub old_lock_file
+{
+return if (!$_[0] || defined($main::locked_file_list{$_[0]}));
+local $pid;
+while(1) {
+	if (!open(LOCKING, ">>$_[0].lock")) {
+		print STDERR "Failed to create lockfile $_[0].lock : $!\n";
+		return;
+		}
+	close(LOCKING);
+	open(LOCKING, "+<$_[0].lock");
+	if (flock(LOCKING, 2+6)) {
+		# got exclusive lock on the lockfile .. check the contents
+		chop($pid = <LOCKING>);
+		if ($pid && kill(0, $pid) && $pid != $$) {
+			# Still locked by another process
+			flock(LOCKING, 8);
+			close(LOCKING);
+			}
+		else {
+			# Got the lock!
+			seek(LOCKING, 0, 0);
+			print LOCKING "$$\n";
+			flock(LOCKING, 8);
+			close(LOCKING);
+			$main::locked_file_list{$_[0]} = int($_[1]);
+			if ($gconfig{'logfiles'} && !$_[1] && !-d $_[0]) {
+				# Grab a copy of this file for later diffing
+				$main::locked_file_data{$_[0]} = undef;
+				local $lnk;
+				if (-d $_[0]) {
+					$main::locked_file_type{$_[0]} = 1;
+					}
+				elsif ($lnk = readlink($_[0])) {
+					$main::locked_file_type{$_[0]} = 2;
+					$main::locked_file_data{$_[0]} = $lnk;
+					}
+				elsif (open(ORIGFILE, $_[0])) {
+					$main::locked_file_type{$_[0]} = 0;
+					$main::locked_file_data{$_[0]} = '';
+					while(<ORIGFILE>) {
+						$main::locked_file_data{$_[0]} .= $_;
+						}
+					close(ORIGFILE);
+					}
+				}
+			last;
+			}
+		}
+	else {
+		# lockfile is itself locked
+		close(LOCKING);
+		}
+	sleep(1);
+	}
 }
 
 # unlock_file(filename)
-# Release a lock grabbed by lock_file()
+# Release a lock on a file. When unlocking a file that was locked in
+# read mode, optionally save the update in RCS
 sub unlock_file
 {
+return if (!$_[0] || !defined($main::locked_file_list{$_[0]}));
+unlink("$_[0].lock");
+delete($main::locked_file_list{$_[0]});
+if (exists($main::locked_file_data{$_[0]})) {
+	# Diff the new file with the old
+	stat($_[0]);
+	local $lnk = readlink($_[0]);
+	local $type = -d _ ? 1 : $lnk ? 2 : 0;
+	local $oldtype = $main::locked_file_type{$_[0]};
+	local $new = !defined($main::locked_file_data{$_[0]});
+	if ($new && !-e _) {
+		# file doesn't exist, and never did! do nothing ..
+		}
+	elsif ($new && $type == 1 || !$new && $oldtype == 1) {
+		# is (or was) a directory ..
+		if (-d _ && !defined($main::locked_file_data{$_[0]})) {
+			push(@main::locked_file_diff,
+			     { 'type' => 'mkdir', 'object' => $_[0] });
+			}
+		elsif (!-d _ && defined($main::locked_file_data{$_[0]})) {
+			push(@main::locked_file_diff,
+			     { 'type' => 'rmdir', 'object' => $_[0] });
+			}
+		}
+	elsif ($new && $type == 2 || !$new && $oldtype == 2) {
+		# is (or was) a symlink ..
+		if ($lnk && !defined($main::locked_file_data{$_[0]})) {
+			push(@main::locked_file_diff,
+			     { 'type' => 'symlink', 'object' => $_[0],
+			       'data' => $lnk });
+			}
+		elsif (!$lnk && defined($main::locked_file_data{$_[0]})) {
+			push(@main::locked_file_diff,
+			     { 'type' => 'unsymlink', 'object' => $_[0],
+			       'data' => $main::locked_file_data{$_[0]} });
+			}
+		elsif ($lnk ne $main::locked_file_data{$_[0]}) {
+			push(@main::locked_file_diff,
+			     { 'type' => 'resymlink', 'object' => $_[0],
+			       'data' => $lnk });
+			}
+		}
+	else {
+		# is a file, or has changed type?!
+		local ($diff, $delete_file);
+		local $type = "modify";
+		if (!-r _) {
+			open(NEWFILE, ">$_[0]");
+			close(NEWFILE);
+			$delete_file++;
+			$type = "delete";
+			}
+		if (!defined($main::locked_file_data{$_[0]})) {
+			$type = "create";
+			}
+		open(ORIGFILE, ">$_[0].webminorig");
+		print ORIGFILE $main::locked_file_data{$_[0]};
+		close(ORIGFILE);
+		$diff = `diff $_[0].webminorig $_[0]`;
+		push(@main::locked_file_diff,
+		     { 'type' => $type, 'object' => $_[0],
+		       'data' => $diff } ) if ($diff);
+		unlink("$_[0].webminorig");
+		unlink($_[0]) if ($delete_file);
+		}
+	delete($main::locked_file_data{$_[0]});
+	delete($main::locked_file_type{$_[0]});
+	}
+}
+
+# unlock_all_files()
+# Unlocks all files locked by this program
+sub unlock_all_files
+{
+foreach $f (keys %main::locked_file_list) {
+	&unlock_file($f);
+	}
+}
+
+# webmin_log(event, type, object, &params, [module])
+# Log some action taken by a user
+sub webmin_log
+{
+return if (!$gconfig{'log'});
+local $m = $_[4] ? $_[4] : $module_name;
+
+if ($gconfig{'logclear'}) {
+	# check if it is time to clear the log
+	local @st = stat("$webmin_logfile.time");
+	local $write_logtime = 0;
+	if (@st) {
+		if ($st[9]+$gconfig{'logtime'}*60*60 < time()) {
+			# clear logfile and all diff files
+			system("rm -f $ENV{'WEBMIN_VAR'}/diffs/* 2>/dev/null");
+			unlink($webmin_logfile);
+			$write_logtime = 1;
+			}
+		}
+	else { $write_logtime = 1; }
+	if ($write_logtime) {
+		open(LOGTIME, ">$webmin_logfile.time");
+		print LOGTIME time(),"\n";
+		close(LOGTIME);
+		}
+	}
+
+# should logging be done at all?
+return if ($gconfig{'logusers'} && &indexof($ENV{'REMOTE_USER'},
+	   split(/\s+/, $gconfig{'logusers'})) < 0);
+return if ($gconfig{'logmodules'} && &indexof($m,
+	   split(/\s+/, $gconfig{'logmodules'})) < 0);
+
+# log the action
+local $now = time();
+local @tm = localtime($now);
+local $script_name = $0 =~ /([^\/]+)$/ ? $1 : '-';
+local $id = sprintf "%d.%d.%d",
+		$now, $$, $main::action_id_count;
+$main::action_id_count++;
+local $line =
+    sprintf "%s [%2.2d/%s/%4.4d %2.2d:%2.2d:%2.2d] %s %s %s %s %s %s %s %s",
+	$id, $tm[3], $make_date_marr[$tm[4]], $tm[5]+1900,
+	$tm[2], $tm[1], $tm[0],
+	$ENV{'REMOTE_USER'}, $ENV{'SESSION_ID'} ? $ENV{'SESSION_ID'} : '-',
+	$ENV{'REMOTE_HOST'},
+	$m, $script_name,
+	$_[0], $_[1] ne '' ? $_[1] : '-', $_[2] ne '' ? $_[2] : '-';
+foreach $k (sort { $a cmp $b } keys %{$_[3]}) {
+	local $v = $_[3]->{$k};
+	if ($v eq '') {
+		$line .= " $k=''";
+		}
+	elsif (ref($v) eq 'ARRAY') {
+		foreach $vv (@$v) {
+			next if (ref($vv));
+			$vv =~ s/(['"\\\r\n\t\%])/sprintf("%%%2.2X",ord($1))/ge;
+			$line .= " $k='$vv'";
+			}
+		}
+	elsif (!ref($v)) {
+		foreach $vv (split(/\0/, $v)) {
+			$vv =~ s/(['"\\\r\n\t\%])/sprintf("%%%2.2X",ord($1))/ge;
+			$line .= " $k='$vv'";
+			}
+		}
+	}
+open(WEBMINLOG, ">>$webmin_logfile");
+print WEBMINLOG $line,"\n";
+close(WEBMINLOG);
+
+if ($gconfig{'logfiles'}) {
+	# Find and record the changes made to any locked files
+	local $i = 0;
+	mkdir("$ENV{'WEBMIN_VAR'}/diffs", 0700);
+	foreach $d (@main::locked_file_diff) {
+		open(DIFFLOG, ">$ENV{'WEBMIN_VAR'}/diffs/$id.$i");
+		print DIFFLOG "$d->{'type'} $d->{'object'}\n";
+		print DIFFLOG $d->{'data'};
+		close(DIFFLOG);
+		$i++;
+		}
+	@main::locked_file_diff = undef;
+	}
+}
+
+# additional_log(type, object, data)
+# Records additional log data for an upcoming call to webmin_log, such
+# as command that was run or SQL that was executed.
+sub additional_log
+{
+if ($gconfig{'logfiles'}) {
+	push(@main::locked_file_diff,
+	     { 'type' => $_[0], 'object' => $_[1], 'data' => $_[2] } );
+	}
+}
+
+# system_logged(command)
+# Just calls the system() function, but also logs the command
+sub system_logged
+{
+local $cmd = join(" ", @_);
+local $amd;
+if ($cmd =~ s/(\s*&\s*)$//) {
+	$and = $1;
+	}
+while($cmd =~ s/(\d*)(<|>)((\/\S+)|&\d+)\s*$//) { }
+$cmd =~ s/^\((.*)\)\s*$/$1/;
+$cmd .= $and;
+&additional_log('exec', undef, $cmd);
+return system(@_);
+}
+
+# backquote_logged(command)
+# Executes a command and returns the output (like `cmd`), but also logs it
+sub backquote_logged
+{
+local $cmd = $_[0];
+local $amd;
+if ($cmd =~ s/(\s*&\s*)$//) {
+	$and = $1;
+	}
+while($cmd =~ s/(\d*)(<|>)((\/\S+)|&\d+)\s*$//) { }
+$cmd =~ s/^\((.*)\)\s*$/$1/;
+$cmd .= $and;
+&additional_log('exec', undef, $cmd);
+return `$_[0]`;
+}
+
+# kill_logged(signal, pid, ...)
+sub kill_logged
+{
+&additional_log('kill', $_[0], join(" ", @_[1..@_-1])) if (@_ > 1);
+return kill(@_);
+}
+
+# rename_logged(old, new)
+sub rename_logged
+{
+&additional_log('rename', $_[0], $_[1]) if ($_[0] ne $_[1]);
+return rename($_[0], $_[1]);
 }
 
 1;  # return true?
