@@ -12,15 +12,23 @@ sub setup_ca
 local $conf = `cat ../acl/openssl.cnf`;
 local $acl = "$config_directory/acl";
 $conf =~ s/DIRECTORY/$acl/g;
+&lock_file("$acl/openssl.cnf");
 open(CONF, ">$acl/openssl.cnf");
 print CONF $conf;
 close(CONF);
+&unlock_file("$acl/openssl.cnf");
+&lock_file("$acl/index.txt");
 open(INDEX, ">$acl/index.txt");
 close(INDEX);
+&unlock_file("$acl/index.txt");
+&lock_file("$acl/serial");
 open(SERIAL, ">$acl/serial");
 print SERIAL "011E\n";
 close(SERIAL);
+&unlock_file("$acl/serial");
+&lock_file("$acl/newcerts");
 mkdir("$acl/newcerts", 0755);
+&unlock_file("$acl/newcerts");
 $miniserv{'ca'} = "$acl/ca.pem";
 }
 
